# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
English-Tagalog Parallel Dataset intended for two tasks:
1. Moral Text Classification
2. Instruction Tuning
"""
import json
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{Catapang:2023,
  author    = {Catapang, Jasper Kyle and Visperas, Moses},
  title     = {Emotion-based Morality in Tagalog and English Scenarios (EMoTES-3K): A Parallel Corpus for Explaining (Im)morality of Actions},
  booktitle = {Proceedings of the Joint 3rd NLP4DH and 8th IWCLUL},
  pages     = {1--6},
  month     = {December 1-3},
  year      = {2023},
  organization = {Association for Computational Linguistics},
}
"""

_DATASETNAME = "emotes_3k"

_DESCRIPTION = """\
This dataset is used on the paper "Emotion-based Morality in Tagalog and English Scenarios (EMoTES-3K): A Parallel Corpus for Explaining (Im)morality of Actions"
This dataset is designed for for two tasks:
1. Moral Text Classification
2. Instruction Tuning
"""

_HOMEPAGE = "https://huggingface.co/datasets/NLPinas/EMoTES-3K"

_LANGUAGES = ["tgl"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)

_LICENSE = Licenses.UNKNOWN.value

_LOCAL = False

_URLS = {
    _DATASETNAME: "https://huggingface.co/datasets/NLPinas/EMoTES-3K/resolve/main/EMoTES-3K.jsonl?download=true",
}

_SUPPORTED_TASKS = [Tasks.MORALITY_CLASSIFICATION, Tasks.INSTRUCTION_TUNING]  # Roberta moral or immoral classification  # FLAN-T5 Training

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class Emotes3KDatasets(datasets.GeneratorBasedBuilder):
    """
    Emotes3K consists of one human annotated dataset for the purpose of morality classification and instruction tuning.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_text",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_text",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_eng_seacrowd_text",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_text",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_tgl_seacrowd_text",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_text",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_t2t",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_t2t",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_eng_seacrowd_t2t",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_t2t",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_tgl_seacrowd_t2t",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_t2t",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "entry_id": datasets.Value("string"),
                    "Filipino": datasets.Value("string"),
                    "English": datasets.Value("string"),
                    "Annotation": datasets.ClassLabel(names=["Moral", "Immoral"]),
                    "Explanation": datasets.Value("string"),
                    "Personality Traits": datasets.Value("string"),
                    "Topic": datasets.Value("string"),
                    "Topic Name": datasets.Value("string"),
                }
            )
        # For example seacrowd_kb, seacrowd_t2t
        elif self.config.schema == "seacrowd_text":
            features = schemas.text.features(["Moral", "Immoral"])
        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text_to_text.features
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        path = dl_manager.download_and_extract(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": path,
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        with open(filepath, "r", encoding="utf-8") as file:
            for line in file:
                # Use json.loads to parse each line as a JSON object
                data = json.loads(line.strip())

                if self.config.schema == "source":
                    yield (
                        data["entry_id"],
                        {
                            "entry_id": data["entry_id"],
                            "Filipino": data["Filipino"],
                            "English": data["English"],
                            "Annotation": data["Annotation"],
                            "Explanation": data["Explanation"],
                            "Personality Traits": data["Personality Traits"],
                            "Topic": data["Topic"],
                            "Topic Name": data["Topic Name"],
                        },
                    )
                elif self.config.schema == "seacrowd_text":
                    if "eng" in self.config.name or self.config.name == "emotes_3k_seacrowd_text":
                        yield (
                            data["entry_id"],
                            {
                                "id": data["entry_id"],
                                "text": data["English"],
                                "label": data["Annotation"],
                            },
                        )
                    elif "tgl" in self.config.name:
                        yield (
                            data["entry_id"],
                            {
                                "id": data["entry_id"],
                                "text": data["Filipino"],
                                "label": data["Annotation"],
                            },
                        )
                elif self.config.schema == "seacrowd_t2t":
                    if "eng" in self.config.name or self.config.name == "emotes_3k_seacrowd_t2t":
                        yield (
                            data["entry_id"],
                            {
                                "id": data["entry_id"],
                                "text_1": "Explain the morality of this scenario\n" + data["English"],
                                "text_2": data["Explanation"],
                                "text_1_name": "prompt",
                                "text_2_name": "system",
                            },
                        )
                    elif "tgl" in self.config.name:
                        yield (
                            data["entry_id"],
                            {
                                "id": data["entry_id"],
                                "text_1": "Explain the morality of this scenario\n" + data["Filipino"],
                                "text_2": data["Explanation"],
                                "text_1_name": "prompt",
                                "text_2_name": "system",
                            },
                        )



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
https://github.com/ir-nlp-csui/indoler/tree/main
The dataset contains 993 annotated court decission document. 
The document was taken from Decision of the Supreme Court of Indonesia. 
The documents have also been tokenized and cleaned
"""
import os
import json
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks, Licenses

_CITATION = """\
@INPROCEEDINGS{9263157,
  author={Nuranti, Eka Qadri and Yulianti, Evi},
  booktitle={2020 International Conference on Advanced Computer Science and Information Systems (ICACSIS)}, 
  title={Legal Entity Recognition in Indonesian Court Decision Documents Using Bi-LSTM and CRF Approaches}, 
  year={2020},
  volume={},
  number={},
  pages={429-434},
  keywords={Xenon;6G mobile communication;legal processing;legal entity recognition;legal document;name entity recognition;ner;bi-lstm;lstm;crf},
  doi={10.1109/ICACSIS51025.2020.9263157}}
"""

_DATASETNAME = "indoler"

_DESCRIPTION = """\
https://github.com/ir-nlp-csui/indoler/tree/main
The data can be used for NER Task in legal documents.
The dataset contains 993 annotated court decission document. 
The document was taken from Decision of the Supreme Court of Indonesia. 
The documents have also been tokenized and cleaned
"""

_HOMEPAGE = "https://github.com/ir-nlp-csui/indoler/tree/main"

_LANGUAGES = ['ind']  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)

_LICENSE = Licenses.UNKNOWN.value 

_LOCAL = False

_URLS = {
    _DATASETNAME: {
        "test_idx": "https://raw.githubusercontent.com/ir-nlp-csui/indoler/main/test.ids.csv",
        "train_idx": "https://raw.githubusercontent.com/ir-nlp-csui/indoler/main/train.ids.csv",
        "valid_idx": "https://raw.githubusercontent.com/ir-nlp-csui/indoler/main/val.ids.csv",
        "full_data": "https://raw.githubusercontent.com/ir-nlp-csui/indoler/main/data.json"
    },
}

_SUPPORTED_TASKS = [Tasks.NAMED_ENTITY_RECOGNITION]  

_SOURCE_VERSION = "2.0.0"

_SEACROWD_VERSION = "1.0.0"



class IndoLer(datasets.GeneratorBasedBuilder):
    """https://github.com/ir-nlp-csui/indoler/tree/main
The data can be used for NER Task in legal documents
The dataset contains 993 annotated court decission document. 
The document was taken from Decision of the Supreme Court of Indonesia. 
The documents have also been tokenized and cleaned"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="indoler_source",
            version=SOURCE_VERSION,
            description="indoler source schema",
            schema="source",
            subset_id="indoler",
        ),
        SEACrowdConfig(
            name="indoler_seacrowd_seq_label",
            version=SEACROWD_VERSION,
            description="indoler SEACrowd schema",
            schema="seacrowd_seq_label",
            subset_id="indoler",
        ),
    ]

    DEFAULT_CONFIG_NAME = "indoler_source"

    def _info(self) -> datasets.DatasetInfo:

        NAMED_ENTITIES = ['O', 'B-Jenis Amar', 'B-Jenis Dakwaan', 'B-Jenis Perkara', 'B-Melanggar UU (Dakwaan)', 
                    'B-Melanggar UU (Pertimbangan Hukum)', 'B-Melanggar UU (Tuntutan)', 'B-Nama Hakim Anggota', 'B-Nama Hakim Ketua', 
                    'B-Nama Jaksa', 'B-Nama Panitera', 'B-Nama Pengacara', 'B-Nama Pengadilan', 
                    'B-Nama Saksi', 'B-Nama Terdakwa', 'B-Nomor Putusan', 'B-Putusan Hukuman', 
                    'B-Tanggal Kejadian', 'B-Tanggal Putusan', 'B-Tingkat Kasus', 'B-Tuntutan Hukuman', 
                    'I-Jenis Amar', 'I-Jenis Dakwaan', 'I-Jenis Perkara', 'I-Melanggar UU (Dakwaan)', 
                    'I-Melanggar UU (Pertimbangan Hukum)', 'I-Melanggar UU (Tuntutan)', 'I-Nama Hakim Anggota', 'I-Nama Hakim Ketua', 
                    'I-Nama Jaksa', 'I-Nama Panitera', 'I-Nama Pengacara', 'I-Nama Pengadilan', 
                    'I-Nama Saksi', 'I-Nama Terdakwa', 'I-Nomor Putusan', 'I-Putusan Hukuman', 
                    'I-Tanggal Kejadian', 'I-Tanggal Putusan', 'I-Tingkat Kasus', 'I-Tuntutan Hukuman']

        if self.config.schema == "source":
            features = datasets.Features({
                "id": datasets.Value("string"),
                "owner": datasets.Value("string"),
                "lawyer": datasets.ClassLabel(names=[False, True]),
                "verdict": datasets.ClassLabel(names=["guilty", "bebas", "lepas"]),
                "indictment": datasets.ClassLabel(names=["NA", "tunggal", "subsider", "komul", "alternatif", "kombinasi", "gabungan"]),
                "text-tags": datasets.Sequence(datasets.ClassLabel(names=NAMED_ENTITIES)),
                "text": datasets.Sequence(datasets.Value("string")),
            })
        elif self.config.schema == "seacrowd_seq_label":
            features = schemas.seq_label.features(NAMED_ENTITIES)
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        test_path = dl_manager.download_and_extract(urls['test_idx'])
        train_path = dl_manager.download_and_extract(urls['train_idx'])
        valid_path = dl_manager.download_and_extract(urls['valid_idx'])
        data_path = dl_manager.download_and_extract(urls['full_data'])
        
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_path,
                    "idx_path": train_path,
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": data_path,
                    "idx_path": test_path,
                    "split": "test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": data_path,
                    "idx_path": valid_path,
                    "split": "validation",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, idx_path: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        split_idxs = []
        with open(idx_path, 'r', encoding="utf-8") as indexes:
            for index in indexes.readlines():
                split_idxs.append(int(index))
        with open(filepath, 'r', encoding="utf-8") as file:
            contents = json.load(file)
            counter = 0
            for content in contents:
                if int(content['id']) in split_idxs:
                    if self.config.schema == "source":
                        if content['indictment'] not in ["NA", "tunggal", "subsider", "komul", "alternatif", "kombinasi", "gabungan"]:
                            content['indictment'] = "NA"
                        yield(
                            counter,
                            {
                                "id"        : content['id'],
                                "owner"     : content['owner'],
                                "lawyer"    : content['lawyer'],
                                "verdict"   : content['verdict'],
                                "indictment": content['indictment'],
                                "text-tags" : content['text-tags'],
                                "text"      : content['text'],
                            }
                        )        
                        counter += 1
                    elif self.config.schema == "seacrowd_seq_label":
                        yield(
                            counter,
                            {
                                "id": content['id'],
                                "tokens": content['text'],
                                "labels": content['text-tags'],
                            }
                        )
                        counter += 1



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from pathlib import Path
from typing import Dict, List, Tuple

import audiosegment
import datasets
import textgrid

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@INPROCEEDINGS{
10337314,
author={Rahim, Mohd Zulhafiz and Juan, Sarah Samson and Mohamad, Fitri Suraya},
booktitle={2023 International Conference on Asian Language Processing (IALP)},
title={Improving Speaker Diarization for Low-Resourced Sarawak Malay Language Conversational Speech Corpus},
year={2023},
pages={228-233},
keywords={Training;Oral communication;Data models;Usability;Speech processing;Testing;Speaker diarization;x-vectors;clustering;low-resource;auto-labeling;pseudo-labeling;unsupervised},
doi={10.1109/IALP61005.2023.10337314}}
"""

_DATASETNAME = "sarawak_malay"

_DESCRIPTION = """\
This is a Sarawak Malay conversation data for the purpose of speech technology research. \
At the moment, this is an experimental data and currently used for investigating \
speaker diarization. The data was collected by Faculty of Computer Science and \
Information Technology, Universiti Malaysia Sarawak. The data consists of 38 conversations \
that have been transcribed using Transcriber (see TextGrid folder), where each file \
contains two speakers. Each conversation was recorded by different individuals using microphones \
from mobile devices or laptops thus, different file formats were collected from the data collectors. \
All data was then standardized to mono, 16000Khz, wav format.
"""

_HOMEPAGE = "https://github.com/sarahjuan/sarawakmalay"

_LANGUAGES = ["zlm"]

_LICENSE = Licenses.CC0_1_0.value
_LOCAL = False

_URLS = {
    _DATASETNAME: "https://github.com/sarahjuan/sarawakmalay/archive/refs/heads/main.zip",
}
_SUPPORTED_TASKS = [Tasks.SPEECH_RECOGNITION, Tasks.TEXT_TO_SPEECH]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class SarawakMalayDataset(datasets.GeneratorBasedBuilder):
    """This is experimental Sarawak Malay conversation data collected by \
        Universiti Malaysia Sarawak for speech technology research, \
        specifically speaker diarization. The data includes 38 conversations, \
        each with two speakers, recorded on various devices and then standardized to mono, \
        16000Khz, wav format."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)
    SEACROWD_SCHEMA_NAME = "sptext"

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "speaker_id": datasets.Value("string"),
                    "path": datasets.Value("string"),
                    "audio": datasets.Audio(sampling_rate=16_000),
                    "text": datasets.Value("string"),
                    "metadata": {
                        "malay_text": datasets.Value("string"),
                    },
                }
            )

        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.speech_text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        urls = _URLS[_DATASETNAME]
        data_dir = dl_manager.download_and_extract(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "sarawakmalay-main"),
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        id_counter = 0
        filenames = filter(lambda x: x.endswith(".wav"), os.listdir(f"{filepath}/wav"))
        filenames = map(lambda x: x.replace(".wav", ""), filenames)

        os.makedirs(f"{filepath}/segmented", exist_ok=True)
        for i, filename in enumerate(filenames):
            info = textgrid.TextGrid.fromFile(f"{filepath}/TextGrid/{filename}.TextGrid")
            if len(info) == 3:
                sarawak_conversation, malay_conversation, speakers = info
            else:
                sarawak_conversation, malay_conversation, speakers, _ = info

            audio_file = audiosegment.from_file(f"{filepath}/wav/{filename}.wav").resample(sample_rate_Hz=16000)

            for sarawak_tg, malay_tg, speaker in zip(sarawak_conversation, malay_conversation, speakers):
                start, end, text = sarawak_tg.minTime, sarawak_tg.maxTime, sarawak_tg.mark
                malay_text = malay_tg.mark
                speaker_id = speaker.mark

                start_sec, end_sec = int(start * 1000), int(end * 1000)
                segment = audio_file[start_sec:end_sec]
                segement_filename = f"{filepath}/segmented/{filename}-{round(start, 0)}-{round(end, 0)}.wav"
                segment.export(segement_filename, format="wav")

                if self.config.schema == "source":
                    yield id_counter, {
                        "id": id_counter,
                        "speaker_id": speaker_id,
                        "path": f"{filepath}/wav/{filename}.wav",
                        "audio": segement_filename,
                        "text": text,
                        "metadata": {
                            "malay_text": malay_text,
                        },
                    }

                elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
                    yield id_counter, {"id": id_counter, "speaker_id": speaker_id, "path": f"{filepath}/wav/{filename}.wav", "audio": segement_filename, "text": text, "metadata": {"speaker_age": None, "speaker_gender": None}}

                id_counter += 1



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """
@inproceedings{,
  author    = {Nguyen, Luan Thanh and Van Nguyen, Kiet and Nguyen,  Ngan Luu-Thuy},
  title     = {Constructive and Toxic Speech Detection for Open-domain Social Media Comments in Vietnamese},
  booktitle = {Advances and Trends in Artificial Intelligence. Artificial Intelligence Practices},
  year      = {2021},
  publisher = {Springer International Publishing},
  address   = {Kuala Lumpur, Malaysia},
  pages     = {572--583},
}
"""

_LOCAL = False
_LANGUAGES = ["vie"]
_DATASETNAME = "uit_victsd"
_DESCRIPTION = """
The UIT-ViCTSD (Vietnamese Constructive and Toxic Speech Detection dataset) is a compilation of 10,000 human-annotated
comments intended for constructive and toxic comments detection. The dataset spans 10 domains, reflecting the diverse topics
and expressions found in social media interactions among Vietnamese users.
"""

_HOMEPAGE = "https://github.com/tarudesu/ViCTSD"
_LICENSE = Licenses.UNKNOWN.value
_URL = "https://huggingface.co/datasets/tarudesu/ViCTSD"


_SUPPORTED_TASKS = [Tasks.INTENT_CLASSIFICATION, Tasks.ABUSIVE_LANGUAGE_PREDICTION]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class UiTViCTSDDataset(datasets.GeneratorBasedBuilder):
    """
    Dataset of Vietnamese social media comments annotated
    for constructiveness and toxicity.
    """

    SUBSETS = ["constructiveness", "toxicity"]
    CLASS_LABELS = [0, 1]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{subset}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME} source schema for {subset} subset",
            schema="source",
            subset_id=f"{_DATASETNAME}_{subset}",
        )
        for subset in SUBSETS
    ] + [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{subset}_seacrowd_text",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME} SEACrowd schema for {subset} subset",
            schema="seacrowd_text",
            subset_id=f"{_DATASETNAME}_{subset}",
        )
        for subset in SUBSETS
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_constructiveness_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "Unnamed: 0": datasets.Value("int64"),  # Column name missing in original dataset
                    "Comment": datasets.Value("string"),
                    "Constructiveness": datasets.ClassLabel(names=self.CLASS_LABELS),
                    "Toxicity": datasets.ClassLabel(names=self.CLASS_LABELS),
                    "Title": datasets.Value("string"),
                    "Topic": datasets.Value("string"),
                }
            )

        elif self.config.schema == "seacrowd_text":
            features = schemas.text_features(label_names=self.CLASS_LABELS)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        # dl_manager not used since dataloader uses HF 'load_dataset'
        return [datasets.SplitGenerator(name=split, gen_kwargs={"split": split._name}) for split in (datasets.Split.TRAIN, datasets.Split.VALIDATION, datasets.Split.TEST)]

    def _load_hf_data_from_remote(self, split: str) -> datasets.DatasetDict:
        """Load dataset from HuggingFace."""
        HF_REMOTE_REF = "/".join(_URL.split("/")[-2:])
        _hf_dataset_source = datasets.load_dataset(HF_REMOTE_REF, split=split)
        return _hf_dataset_source

    def _generate_examples(self, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        data = self._load_hf_data_from_remote(split=split)
        for index, row in enumerate(data):
            if self.config.schema == "source":
                example = row

            elif self.config.schema == "seacrowd_text":
                if "constructiveness" in self.config.name:
                    label = row["Constructiveness"]
                elif "toxicity" in self.config.name:
                    label = row["Toxicity"]
                example = {"id": str(index), "text": row["Comment"], "label": label}
            yield index, example



from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses

_CITATION = """\
@misc{beayelexicon2024,
  author    = {Lopo, Joanito Agili and Moeljadi, David and Cahyawijaya, Samuel and Aji, Alham Fikri and Sommerlot, 
  Carly J. and Jacob, June},
  title     = {Penyusunan Korpus Paralel Bahasa Indonesia–Bahasa Melayu Ambon, Melayu Kupang, Beaye, dan Uab Meto},
  year      = {2024},
  howpublished = {Online},
  url       = {https://github.com/joanitolopo/makalah-kongresxii},
  note      = {Manuscript in preparation},
}
"""

_DATASETNAME = "beaye_lexicon"
_DESCRIPTION = """The Beaye Lexicon is a lexicon resource encompassing translations between Indonesian, English, and 
Beaye words. Developed through a collaborative effort involving two native Beaye speakers and evaluated by linguistic 
experts, this lexicon comprises 984 Beaye vocabularies. The creation of the Beaye Lexicon marks the inaugural effort in 
documenting the previously unrecorded Beaye language."""

_HOMEPAGE = "https://github.com/joanitolopo/bhinneka-korpus/tree/main/lexicon"
_LICENSE = Licenses.APACHE_2_0.value
_URLS = "https://raw.githubusercontent.com/joanitolopo/bhinneka-korpus/main/lexicon"
_SUPPORTED_TASKS = []
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"
_LOCAL = False

_LANGUAGES = ["ind", "day", "eng"]

class BeayeLexicon(datasets.GeneratorBasedBuilder):
    """Beaye Lexicon is a lexicon resource encompassing translations between Indonesian, English, and Beaye words"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = (
        [
            SEACrowdConfig(
                name=f"{_DATASETNAME}_{lang}_source",
                version=datasets.Version(_SOURCE_VERSION),
                description=f"beaye lexicon with source schema for {lang} language",
                schema="source",
                subset_id="beaye_lexicon",
            )
            for lang in _LANGUAGES if lang != "eng"
        ]
        + [
            SEACrowdConfig(
                name=f"{_DATASETNAME}_ext_{lang}_source",
                version=datasets.Version(_SOURCE_VERSION),
                description=f"beaye lexicon with source schema for extensive definiton of beaye language",
                schema="source",
                subset_id="beaye_lexicon",
            )
            for lang in _LANGUAGES if lang != "ind"
        ]
    )

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_ind_source"

    def _info(self) -> datasets.DatasetInfo:
        schema = self.config.schema
        if schema == "source":
            features = datasets.Features({"id": datasets.Value("string"), "word": datasets.Value("string")})
        else:
            raise NotImplementedError()

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        if "ext" in self.config.name.split("_"):
            data_dir = Path(dl_manager.download(_URLS + "/english.xlsx"))
        else:
            data_dir = Path(dl_manager.download(_URLS + "/lexicon.xlsx"))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir,
                    "split": "train",
                }
            )
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        dfs = pd.read_excel(filepath, engine="openpyxl")
        if "ext" in self.config.name.split("_"):
            lang = self.config.name.split("_")[3]
        else:
            lang = self.config.name.split("_")[2]

        text = dfs[lang]

        if self.config.schema == "source":
            for idx, word in enumerate(text.values):
                row = {"id": str(idx), "word": word}
                yield idx, row
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks
from seacrowd.utils import schemas
import jsonlines
from nltk.tokenize.treebank import TreebankWordDetokenizer

_CITATION = """\
@INPROCEEDINGS{8629109,
  author={Kurniawan, Kemal and Louvan, Samuel},
  booktitle={2018 International Conference on Asian Language Processing (IALP)}, 
  title={Indosum: A New Benchmark Dataset for Indonesian Text Summarization}, 
  year={2018},
  volume={},
  number={},
  pages={215-220},
  doi={10.1109/IALP.2018.8629109}}
"""

_LOCAL = False
_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_DATASETNAME = "indosum"

_DESCRIPTION = """\
INDOSUM is a new benchmark dataset for Indonesian text summarization. 
The dataset consists of news articles and manually constructed summaries.
"""

_HOMEPAGE = "https://github.com/kata-ai/indosum"

_LICENSE = "Apache License, Version 2.0"

_URLS = {
    _DATASETNAME: "https://drive.google.com/uc?id=1OgYbPfXFAv3TbwP1Qcwt_CC9cVWSJaco",
}

_SUPPORTED_TASKS = [Tasks.SUMMARIZATION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class IndoSUM(datasets.GeneratorBasedBuilder):
    """INDOSUM is a new benchmark dataset for Indonesian text summarization. The dataset consists of news articles and manually constructed summaries."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = (
        [
            SEACrowdConfig(
                name="indosum_fold{fold_number}_source".format(fold_number=i),
                version=_SOURCE_VERSION,
                description="indosum source schema",
                schema="source",
                subset_id="indosum_fold{fold_number}".format(fold_number=i),
            ) for i in range(5)
        ]
        +
        [
            SEACrowdConfig(
            name="indosum_fold{fold_number}_seacrowd_t2t".format(fold_number=i),
            version=_SEACROWD_VERSION,
            description="indosum Nusantara schema",
            schema="seacrowd_t2t",
            subset_id="indosum_fold{fold_number}".format(fold_number=i),
        ) for i in range(5)
        ]
    )

    DEFAULT_CONFIG_NAME = "indosum_fold0_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":

            features = datasets.Features(
               {
                   "document": datasets.Value("string"),
                   "id": datasets.Value("string"),
                   "summary": datasets.Value("string")
               }
            )

        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _get_fold_index(self):
        try:
            subset_id = self.config.subset_id
            idx_fold = subset_id.index("_fold")
            file_id = subset_id[(idx_fold + 5):]
            return int(file_id)
        except:
            return 0

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        idx = self._get_fold_index()

        urls = _URLS[_DATASETNAME]

        data_dir = Path(dl_manager.download_and_extract(urls))

        location = {
            "train": "indosum/train.0{fold_number}.jsonl",
            "test": "indosum/test.0{fold_number}.jsonl",
            "dev": "indosum/dev.0{fold_number}.jsonl"
        }

        data_dir = dl_manager.download_and_extract(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,

                gen_kwargs={
                    "filepath": os.path.join(data_dir, location["train"].format(fold_number=idx+1)),
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, location["test"].format(fold_number=idx+1)),
                    "split": "test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, location["dev"].format(fold_number=idx+1)),
                    "split": "dev",
                },
            ),
        ]

    def _get_full_paragraph_and_summary(self, data: Dict) -> Tuple[str, str]:
        detokenizer = TreebankWordDetokenizer()
        paragraph = ""
        summary = ""
        begin_paragraph = True
        begin_summary = True

        for each_paragraph in data["paragraphs"]:
            for each_sentence in each_paragraph:
                detokenized_sentence = detokenizer.detokenize(each_sentence)
                if begin_paragraph:
                    paragraph+=detokenized_sentence
                    begin_paragraph = False
                else:
                    paragraph = "{} {}".format(paragraph, detokenized_sentence)
        
        for each_summary in data["summary"]:
            detokenized_sentence = detokenizer.detokenize(each_summary)
            if begin_summary:
                summary+=detokenized_sentence
                begin_summary = False
            else:
                summary = "{} {}".format(summary, detokenized_sentence)

        return paragraph, summary

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:

        if self.config.schema == "source":
            i = 0
            with jsonlines.open(filepath) as f:
                for each_data in f.iter():
                    full_paragraph, full_summary = self._get_full_paragraph_and_summary(each_data)
                    ex = {
                        "id": each_data["id"],
                        "document": full_paragraph,
                        "summary": full_summary
                    }
                    yield i, ex
                    i+=1

        elif self.config.schema == "seacrowd_t2t":
            i = 0
            with jsonlines.open(filepath) as f:
                for each_data in f.iter():
                    full_paragraph, full_summary = self._get_full_paragraph_and_summary(each_data)
                    ex = {
                        "id": each_data["id"],
                        "text_1": full_paragraph,
                        "text_2": full_summary,
                        "text_1_name": "document",
                        "text_2_name": "summary"
                    }
                    yield i, ex
                    i+=1



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
""" IndoNLU NERGrit Dataset """

from pathlib import Path
from typing import List

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.common_parser import load_conll_data
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@inproceedings{wilie2020indonlu,
  title={IndoNLU: Benchmark and Resources for Evaluating Indonesian Natural Language Understanding},
  author={Bryan Wilie and Karissa Vincentio and Genta Indra Winata and Samuel Cahyawijaya and X. Li and Zhi Yuan Lim and S. Soleman and R. Mahendra and Pascale Fung and Syafri Bahar and A. Purwarianti},
  booktitle={Proceedings of the 1st Conference of the Asia-Pacific Chapter of the Association for Computational Linguistics and the 10th International Joint Conference on Natural Language Processing},
  year={2020}
}
@online{nergrit2019,
  title={NERGrit Corpus},
  author={NERGrit Developers},
  year={2019},
  url={https://github.com/grit-id/nergrit-corpus}
}
"""

_LOCAL = False
_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_DATASETNAME = "indonlu_nergrit"
_DESCRIPTION = """\
This NER dataset is taken from the Grit-ID repository, and the labels are spans in IOB chunking representation.
The dataset consists of three kinds of named entity tags, PERSON (name of person), PLACE (name of location), and
ORGANIZATION (name of organization).
"""

_HOMEPAGE = "https://github.com/grit-id/nergrit-corpus"
_LICENSE = "MIT"
_URL_ROOT = "https://raw.githubusercontent.com/IndoNLP/indonlu/master/dataset/nergrit_ner-grit"
_URLs = {
    "train": f"{_URL_ROOT}/train_preprocess.txt",
    "validation": f"{_URL_ROOT}/valid_preprocess.txt",
    "test": f"{_URL_ROOT}/test_preprocess.txt",
}
_SUPPORTED_TASKS = [Tasks.NAMED_ENTITY_RECOGNITION]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class IndonluNergritDataset(datasets.GeneratorBasedBuilder):
    """Indonesian Named Entity Recognition from https://github.com/grit-id/nergrit-corpus."""

    label_classes = ["I-PERSON", "B-ORGANISATION", "I-ORGANISATION", "B-PLACE", "I-PLACE", "O", "B-PERSON"]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="indonlu_nergrit_source",
            version=datasets.Version(_SOURCE_VERSION),
            description="IndoNLU NERGrit source schema",
            schema="source",
            subset_id="indonlu_nergrit",
        ),
        SEACrowdConfig(
            name="indonlu_nergrit_seacrowd_seq_label",
            version=datasets.Version(_SEACROWD_VERSION),
            description="IndoNLU NERGrit Nusantara schema",
            schema="seacrowd_seq_label",
            subset_id="indonlu_nergrit",
        ),
    ]

    DEFAULT_CONFIG_NAME = "indonlu_nergrit_source"

    def _info(self):
        features = None
        if self.config.schema == "source":
            features = datasets.Features({"index": datasets.Value("string"), "tokens": [datasets.Value("string")], "ner_tag": [datasets.Value("string")]})
        elif self.config.schema == "seacrowd_seq_label":
            features = schemas.seq_label_features(self.label_classes)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        train_tsv_path = Path(dl_manager.download_and_extract(_URLs["train"]))
        validation_tsv_path = Path(dl_manager.download_and_extract(_URLs["validation"]))
        test_tsv_path = Path(dl_manager.download_and_extract(_URLs["test"]))
        data_files = {
            "train": train_tsv_path,
            "validation": validation_tsv_path,
            "test": test_tsv_path,
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_files["train"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": data_files["validation"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": data_files["test"]},
            ),
        ]

    def _generate_examples(self, filepath: Path):
        conll_dataset = load_conll_data(filepath)

        if self.config.schema == "source":
            for index, row in enumerate(conll_dataset):
                ex = {"index": str(index), "tokens": row["sentence"], "ner_tag": row["label"]}
                yield index, ex
        elif self.config.schema == "seacrowd_seq_label":
            for index, row in enumerate(conll_dataset):
                ex = {"id": str(index), "tokens": row["sentence"], "labels": row["label"]}
                yield index, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import csv
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses

_CITATION = """\
@article{article,
author = {Borra, Allan and Pease, Adam and Edita, Rachel and Roxas, and Dita, Shirley},
year = {2010},
month = {01},
pages = {},
title = {Introducing Filipino WordNet}
}
"""

_DATASETNAME = "filwordnet"

_DESCRIPTION = """\
Filipino WordNet (FilWordNet) is a lexical database of Filipino language.
It was derived from the Princeton WordNet and translated by humans to Filipino.
It documents 13,539 unique words and 9,519 synsets. Each synset includes the definition,
part-of-speech, word senses, and Suggested Upper Merged Ontology terms (SUMO terms).
"""

_HOMEPAGE = "https://github.com/danjohnvelasco/Filipino-WordNet"

_LANGUAGES = ["fil"]

_LICENSE = Licenses.UNKNOWN.value

_LOCAL = False

_URLS = {
    _DATASETNAME: "https://raw.githubusercontent.com/danjohnvelasco/Filipino-WordNet/main/filwordnet.csv",
}

_SUPPORTED_TASKS = []

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class FilWordNetDataset(datasets.GeneratorBasedBuilder):
    """The Filipino WordNet (FilWordNet) is a lexical database of Filipino language containing 13,539 unique words and 9,519 synsets."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        )
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "word_id": datasets.Value("int32"),
                    "lemma": datasets.Value("string"),
                    "synset_id": datasets.Value("int32"),
                    "sense_id": datasets.Value("int32"),
                    "pos": datasets.Value("string"),
                    "lexdomain_id": datasets.Value("int32"),
                    "definition": datasets.Value("string"),
                    "last_modifier": datasets.Value("int32"),
                    "sumo": datasets.Value("string"),
                }
            )

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        file = dl_manager.download_and_extract(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": file,
                    "split": "train",
                },
            )
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        rows = []
        is_first_row = True
        with open(filepath, "r") as file:
            csv_reader = csv.reader(file, delimiter=",")
            for row in csv_reader:
                if is_first_row:  # skip first row, they are column names
                    is_first_row = False
                    continue

                rows.append(row)

        if self.config.schema == "source":
            for key, row in enumerate(rows):
                example = {
                    "word_id": row[0],
                    "lemma": row[1],
                    "synset_id": row[2],
                    "sense_id": row[3],
                    "pos": row[4],
                    "lexdomain_id": row[5],
                    "definition": row[6],
                    "last_modifier": row[7],
                    "sumo": row[8],
                }
                yield key, example



import csv
import os
from typing import Dict, List

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import (DEFAULT_SEACROWD_VIEW_NAME,
                                       DEFAULT_SOURCE_VIEW_NAME, Tasks)

_DATASETNAME = "su_id_asr"
_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME

_LANGUAGES = ["sun"]
_LOCAL = False
_CITATION = """\
@inproceedings{sodimana18_sltu,
  author={Keshan Sodimana and Pasindu {De Silva} and Supheakmungkol Sarin and Oddur Kjartansson and Martin Jansche and Knot Pipatsrisawat and Linne Ha},
  title={{A Step-by-Step Process for Building TTS Voices Using Open Source Data and Frameworks for Bangla, Javanese, Khmer, Nepali, Sinhala, and Sundanese}},
  year=2018,
  booktitle={Proc. 6th Workshop on Spoken Language Technologies for Under-Resourced Languages (SLTU 2018)},
  pages={66--70},
  doi={10.21437/SLTU.2018-14}
}
"""

_DESCRIPTION = """\
Sundanese ASR training data set containing ~220K utterances.
This dataset was collected by Google in Indonesia.


"""

_HOMEPAGE = "https://indonlp.github.io/nusa-catalogue/card.html?su_id_asr"

_LICENSE = "Attribution-ShareAlike 4.0 International."

_URLs = {
    "su_id_asr": "https://www.openslr.org/resources/36/asr_sundanese_{}.zip",
}

_SUPPORTED_TASKS = [Tasks.SPEECH_RECOGNITION]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class SuIdASR(datasets.GeneratorBasedBuilder):
    """su_id contains ~220K utterances for Sundanese ASR training data."""

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="su_id_asr_source",
            version=datasets.Version(_SOURCE_VERSION),
            description="SU_ID_ASR source schema",
            schema="source",
            subset_id="su_id_asr",
        ),
        SEACrowdConfig(
            name="su_id_asr_seacrowd_sptext",
            version=datasets.Version(_SEACROWD_VERSION),
            description="SU_ID_ASR Nusantara schema",
            schema="seacrowd_sptext",
            subset_id="su_id_asr",
        ),
    ]

    DEFAULT_CONFIG_NAME = "su_id_asr_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "speaker_id": datasets.Value("string"),
                    "path": datasets.Value("string"),
                    "audio": datasets.Audio(sampling_rate=16_000),
                    "text": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_sptext":
            features = schemas.speech_text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
            task_templates=[datasets.AutomaticSpeechRecognition(audio_column="audio", transcription_column="text")],
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        base_path = {}
        for id in range(10):
            base_path[id] = dl_manager.download_and_extract(_URLs["su_id_asr"].format(str(id)))
        for id in ["a", "b", "c", "d", "e", "f"]:
            base_path[id] = dl_manager.download_and_extract(_URLs["su_id_asr"].format(str(id)))
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": base_path},
            ),
        ]

    def _generate_examples(self, filepath: Dict):

        if self.config.schema == "source" or self.config.schema == "seacrowd_sptext":

            for key, each_filepath in filepath.items():

                tsv_file = os.path.join(each_filepath, "asr_sundanese", "utt_spk_text.tsv")

                with open(tsv_file, "r") as file:
                    tsv_file = csv.reader(file, delimiter="\t")

                    for line in tsv_file:
                        audio_id, speaker_id, transcription_text = line[0], line[1], line[2]

                        wav_path = os.path.join(each_filepath, "asr_sundanese", "data", "{}".format(audio_id[:2]), "{}.flac".format(audio_id))

                        if os.path.exists(wav_path):
                            if self.config.schema == "source":
                                ex = {
                                    "id": audio_id,
                                    "speaker_id": speaker_id,
                                    "path": wav_path,
                                    "audio": wav_path,
                                    "text": transcription_text,
                                }
                                yield audio_id, ex
                            elif self.config.schema == "seacrowd_sptext":
                                ex = {
                                    "id": audio_id,
                                    "speaker_id": speaker_id,
                                    "path": wav_path,
                                    "audio": wav_path,
                                    "text": transcription_text,
                                    "metadata": {
                                        "speaker_age": None,
                                        "speaker_gender": None,
                                    },
                                }
                                yield audio_id, ex

        else:
            raise ValueError(f"Invalid config: {self.config.name}")



import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks
from seacrowd.utils import schemas
import csv

_CITATION = """\
@article{nurlaila2018classification,
  title={CLASSIFICATION OF CUSTOMERS EMOTION USING NA{\"I}VE BAYES CLASSIFIER (Case Study: Natasha Skin Care)},
  author={Nurlaila, Afifah and Wiranto, Wiranto and Saptono, Ristu},
  journal={ITSMART: Jurnal Teknologi dan Informasi},
  volume={6},
  number={2},
  pages={92--97},
  year={2018}
}
"""

_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False

_DATASETNAME = "sentiment_nathasa_review"

_DESCRIPTION = """\
Customer Review (Natasha Skincare) is a customers emotion dataset, with amounted to 19,253 samples with the division for each class is 804 joy, 43 surprise, 154 anger, 61 fear, 287 sad, 167 disgust, and 17736 no-emotions.
"""

_HOMEPAGE = "https://jurnal.uns.ac.id/itsmart/article/viewFile/17328/15082"

_LICENSE = "Unknown"

_URLS = {
    _DATASETNAME: "https://drive.google.com/uc?id=1D1pHX7CxrI-eIl2bAvIp1bWQeucyUGw0",
}

_SUPPORTED_TASKS = [Tasks.SENTIMENT_ANALYSIS]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

class SentimentNathasaReview(datasets.GeneratorBasedBuilder):
    """Customer Review (Natasha Skincare) is a customers emotion dataset, with amounted to 19,253 samples with the division for each class is 804 joy, 43 surprise, 154 anger, 61 fear, 287 sad, 167 disgust, and 17736 no-emotions."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="sentiment_nathasa_review_source",
            version=datasets.Version(_SOURCE_VERSION),
            description="sentiment_nathasa_review source schema",
            schema="source",
            subset_id="sentiment_nathasa_review",
        ),
        SEACrowdConfig(
            name="sentiment_nathasa_review_seacrowd_text",
            version=datasets.Version(_SEACROWD_VERSION),
            description="sentiment_nathasa_review Nusantara schema",
            schema="seacrowd_text",
            subset_id="sentiment_nathasa_review",
        ),
    ]

    DEFAULT_CONFIG_NAME = "sentiment_nathasa_review_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
               {
                    "id": datasets.Value("string"),
                    "usr": datasets.Value("string"),
                    "text": datasets.Value("string"),
                    "label": datasets.Value("string"),
               }
            )
        elif self.config.schema == "seacrowd_text":
            features = schemas.text_features(['NOEMOTION', 'SURPRISE', 'SAD', 'JOY', 'FEAR', 'DISGUST', 'ANGER'])

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )


    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        data_dir = Path(dl_manager.download(_URLS[_DATASETNAME]))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir,
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": data_dir,
                    "split": "test",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        if self.config.schema == "source":
            with open(filepath, "r") as F:
                csvreader = csv.reader(F)
                for row in csvreader:
                    try:
                        row_data = eval(row[0].replace(';',','))[0]
                    except:
                        continue
                    if split == "train" and row_data[3] == "DATA LATIH":
                        ex = {
                            "id": row_data[0],
                            "usr": row_data[1],
                            "text": row_data[4],
                            "label": row_data[2],
                        }
                        yield row_data[0], ex
                    elif split == "test" and row_data[3] == "DATA UJI":
                        ex = {
                            "id": row_data[0],
                            "usr": row_data[1],
                            "text": row_data[4],
                            "label": row_data[2],
                        }
                        yield row_data[0], ex

        elif self.config.schema == "seacrowd_text":
            with open(filepath, "r") as F:
                csvreader = csv.reader(F)
                for row in csvreader:
                    try:
                        row_data = eval(row[0].replace(';',','))[0]
                    except:
                        continue
                    if split == "train" and row_data[3] == "DATA LATIH":
                        ex = {
                            "id": row_data[0],
                            "text": row_data[4],
                            "label": row_data[2],
                        }
                        yield row_data[0], ex
                    elif split == "test" and row_data[3] == "DATA UJI":
                        ex = {
                            "id": row_data[0],
                            "text": row_data[4],
                            "label": row_data[2],
                        }
                        yield row_data[0], ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



import json
import os
from typing import Dict, List, Tuple

# import csv
import datasets
import jsonlines as jl
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{thapliyal-etal-2022-crossmodal,
    title = "Crossmodal-3600: A Massively Multilingual Multimodal Evaluation Dataset",
    author = "Thapliyal, Ashish V.  and
      Pont Tuset, Jordi  and
      Chen, Xi  and
      Soricut, Radu",
    editor = "Goldberg, Yoav  and
      Kozareva, Zornitsa  and
      Zhang, Yue",
    booktitle = "Proceedings of the 2022 Conference on Empirical Methods in Natural Language Processing",
    month = dec,
    year = "2022",
    address = "Abu Dhabi, United Arab Emirates",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2022.emnlp-main.45",
    doi = "10.18653/v1/2022.emnlp-main.45",
    pages = "715--729",
}
"""

_DATASETNAME = "coco_35l"

_DESCRIPTION = """\
    COCO-35L is a machine-generated image caption dataset, constructed by translating COCO Captions (Chen et al., 2015) to the other 34 languages using Google’s machine translation API.
    152520 image ids are not found in the coco 2014 training caption. Validation set is ok Using COCO 2014 train and validation set.
    """

_HOMEPAGE = "https://google.github.io/crossmodal-3600/"

_LICENSE = Licenses.CC_BY_4_0.value

_URLS = {
    "coco2017_train_images": "http://images.cocodataset.org/zips/train2017.zip",
    "coco2014_train_images": "http://images.cocodataset.org/zips/train2014.zip",
    "coco2014_val_images": "http://images.cocodataset.org/zips/val2014.zip",
    "coco2014_train_val_annots": "http://images.cocodataset.org/annotations/annotations_trainval2014.zip",
    "coco2017_train_val_annots": "http://images.cocodataset.org/annotations/annotations_trainval2017.zip",
    "trans_train": "https://storage.googleapis.com/crossmodal-3600/coco_mt_train.jsonl.gz",
    "trans_dev": "https://storage.googleapis.com/crossmodal-3600/coco_mt_dev.jsonl.gz",
}

_SUPPORTED_TASKS = [Tasks.IMAGE_CAPTIONING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

_LANGUAGES = {"fil": "fil", "ind": "id", "tha": "th", "vie": "vi"}

_LOCAL = False

class Coco35LDataset(datasets.GeneratorBasedBuilder):
    """
    COCO-35L is a machine-generated image caption dataset, constructed by translating COCO Captions (Chen et al., 2015) to the other 34 languages using Google’s machine translation API.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{lang}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME}_{lang} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}_{lang}",
        ) for lang in _LANGUAGES
    ] + [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{lang}_seacrowd_imtext",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME}_{lang} SEACrowd schema",
            schema="seacrowd_imtext",
            subset_id=f"{_DATASETNAME}_{lang}",
        ) for lang in _LANGUAGES
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_{sorted(_LANGUAGES)[0]}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "image_paths": datasets.Value("string"),
                    "src_lang": datasets.Value("string"),
                    "caption_tokenized": datasets.Value("string"),
                    "trg_lang": datasets.Value("string"),
                    "translation_tokenized": datasets.Value("string"),
                    "backtranslation_tokenized": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_imtext":
            features = schemas.image_text_features()

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        trans_train_path = dl_manager.download_and_extract(_URLS["trans_train"])
        trans_val_path = dl_manager.download_and_extract(_URLS["trans_dev"])

        coco2014_train_val_annots_path = dl_manager.download_and_extract(_URLS["coco2014_train_val_annots"])
        coco2014_val_images_path = dl_manager.download_and_extract(_URLS["coco2014_val_images"])
        coco2014_train_images_path = dl_manager.download_and_extract(_URLS["coco2014_train_images"])

        trans_train_captions = {}
        trans_dev_captions = {}
        train_df = pd.DataFrame()
        val_df = pd.DataFrame()

        current_lang = _LANGUAGES[self.config.subset_id.split("_")[2]]

        # the COCO dataset structure has separated the captions and images information. The caption's "image_id" key will refer to the image's "id" key.
        # load the image informations from COCO 2014 dataset and put it into a dataframe
        with open(os.path.join(coco2014_train_val_annots_path, "annotations", "captions_val2014.json")) as json_captions:
            captions = json.load(json_captions)
            val_df = pd.DataFrame(captions["images"])

        with open(os.path.join(coco2014_train_val_annots_path, "annotations", "captions_train2014.json")) as json_captions:
            captions = json.load(json_captions)
            train_df = pd.DataFrame(captions["images"])

        # the translated caption has "image_id" which refers to the "image_id" in the COCO annotations.
        # however we can skip this and connect it to the images' "id"
        # the example of an "image_id" in the translated caption -> "123456_0" since an image can has many descriptions.
        # thus, the real image_id to map it into the COCO image dataset is the "123456"
        with jl.open(trans_train_path, mode="r") as j:
            total = 0
            not_found = 0
            missing_ids = []
            for line in j:
                if line["trg_lang"] == current_lang:
                    total += 1

                    trans_img_id = line["image_id"]
                    coco2014_img_id = line["image_id"].split("_")[0]

                    # unfortunately, not all image_id in the translated caption can be found in the original COCO 2014.
                    # hence, we need to handle such errors
                    try:
                        filename = train_df.query(f"id=={int(coco2014_img_id)}")["file_name"].values[0]
                        trans_train_captions[trans_img_id] = line
                        trans_train_captions[trans_img_id]["filename"] = os.path.join(coco2014_train_images_path, "train2014", filename)
                    except IndexError:
                        missing_ids.append(trans_img_id)
                        not_found += 1
                        pass

        # the validation set are strangely okay. with no missing image_id(s)
        with jl.open(trans_val_path, mode="r") as j:
            for line in j:
                if line["trg_lang"] == current_lang:
                    trans_img_id = line["image_id"]
                    trans_dev_captions[trans_img_id] = line
                    coco2014_img_id = int(trans_img_id.split("_")[0])
                    filename = val_df.query(f"id=={coco2014_img_id}")["file_name"].values[0]
                    trans_dev_captions[trans_img_id]["filename"] = os.path.join(coco2014_val_images_path, "val2014", filename)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": {
                        "images": trans_train_captions,
                    },
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": {
                        "images": trans_dev_captions,
                    },
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, filepath: dict, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        counter = 0
        for trans_img_id, data in filepath["images"].items():
            if self.config.schema == "source":
                yield counter, {
                    "id": trans_img_id + "_" + str(counter),
                    "image_paths": data["filename"],
                    "src_lang": data["src_lang"],
                    "caption_tokenized": data["caption_tokenized"],
                    "trg_lang": data["trg_lang"],
                    "translation_tokenized": data["translation_tokenized"],
                    "backtranslation_tokenized": data["backtranslation_tokenized"],
                }

            elif self.config.schema == "seacrowd_imtext":
                yield counter, {
                    "id": trans_img_id + "_" + str(counter),
                    "image_paths": [data["filename"]],
                    "texts": data["translation_tokenized"],
                    "metadata": {
                        "context": None,
                        "labels": None,
                    },
                }

            else:
                raise ValueError(f"Invalid config: {self.config.name}")

            counter += 1



import os
from pathlib import Path
from typing import Dict, List, Tuple
from seacrowd.utils.constants import Tasks
from seacrowd.utils import schemas

import datasets
import json

from seacrowd.utils.configs import SEACrowdConfig

_CITATION = """\
@article{kurniawan2019,
  title={KaWAT: A Word Analogy Task Dataset for Indonesian},
  url={http://arxiv.org/abs/1906.09912},
  journal={arXiv:1906.09912 [cs]},
  author={Kurniawan, Kemal},
  year={2019},
  month={Jun}
}
"""

_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False

_DATASETNAME = "kawat"

_DESCRIPTION = """\
We introduced KaWAT (Kata Word Analogy  Task), a new word analogy task dataset for Indonesian. 
We evaluated on it several existing pretrained Indonesian word embeddings and embeddings trained on Indonesian online news corpus. 
We also tested them on two downstream tasks and found that pretrained word embeddings helped either by reducing the training epochs
or yielding significant performance gains.
"""

_HOMEPAGE = "https://github.com/kata-ai/kawat"

_LICENSE = "Creative Commons Attribution-ShareAlike 4.0"

_URLS = {
    _DATASETNAME: "https://raw.githubusercontent.com/kata-ai/kawat/master/{}/{}",
}

_SUPPORTED_TASKS = [Tasks.WORD_ANALOGY]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

_PATH_FILE = [
    {
        "folder": "semantic",
        "file": [
            "antonyms.txt",
            "country-capitals.txt",
            "country-currencies.txt",
            "gender-specific-words.txt",
            "measure-words.txt",
            "province-capitals.txt"
        ]
    },
    {
        "folder": "syntax",
        "file": [
            "nouns.txt",
            "plurals.txt",
            "reduplications.txt",
            "verbs.txt"
        ]
    }
]

class Kawat(datasets.GeneratorBasedBuilder):

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="kawat_source",
            version=SOURCE_VERSION,
            description="Kawat source schema",
            schema="source",
            subset_id="kawat",
        ),
        SEACrowdConfig(
            name="kawat_seacrowd_t2t",
            version=SEACROWD_VERSION,
            description="Kawat Nusantara schema",
            schema="seacrowd_t2t",
            subset_id="kawat",
        ),
    ]

    DEFAULT_CONFIG_NAME = "kawat_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "text_1": datasets.Value("string"),
                    "text_1_name": datasets.Value("string"),
                    "text_2": datasets.Value("string"),
                    "text_2_name": datasets.Value("string"),
                }
            )

        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        datas = []

        num = 0

        for each_path_file in _PATH_FILE:
            for each_file in each_path_file["file"]:
                data_dir = dl_manager.download_and_extract(_URLS[_DATASETNAME].format(each_path_file['folder'], each_file))

                parsed_lines = open(data_dir, "r").readlines()

                titles = parsed_lines[0].split("\t")

                num_columns = len(titles)

                titles[num_columns-1] = titles[num_columns-1][:-1]

                for i in range(1, len(parsed_lines)):
                    words = parsed_lines[i].split("\t")

                    words[num_columns-1] = words[num_columns-1][:-1]

                    for j in range(1, num_columns):
                        if words[j] != "-":
                            datas.append({
                                "id": str(num),
                                "text_1": words[0],
                                "text_1_name": titles[0],
                                "text_2": words[j],
                                "text_2_name": titles[j],
                            })
                        num+=1
        
        with open(data_dir, 'w') as f:
            f.write(json.dumps(datas))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir,
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        data = json.load(open(filepath, "r"))

        if self.config.schema == "source":
            key = 0
            for each_data in data:
                example = {
                    "id": each_data["id"],
                    "text_1": each_data["text_1"],
                    "text_1_name": each_data["text_1_name"],
                    "text_2": each_data["text_2"],
                    "text_2_name": each_data["text_2_name"],
                }
                yield key, example
                key+=1

        elif self.config.schema == "seacrowd_t2t":
            key = 0
            for each_data in data:
                example = {
                    "id": each_data["id"],
                    "text_1": each_data["text_1"],
                    "text_1_name": each_data["text_1_name"],
                    "text_2": each_data["text_2"],
                    "text_2_name": each_data["text_2_name"],
                }
                yield key, example
                key+=1



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Belebele is a multiple-choice machine reading comprehension (MRC) dataset spanning 122 language variants. 
This dataset enables the evaluation of mono- and multi-lingual models in high-, medium-, and low-resource languages. 
Each question has four multiple-choice answers and is linked to a short passage from the FLORES-200 dataset. 
The human annotation procedure was carefully curated to create questions that discriminate between different
 levels of generalizable language comprehension and is reinforced by extensive quality checks. While all 
 questions directly relate to the passage, the English dataset on its own proves difficult enough to 
 challenge state-of-the-art language models. Being fully parallel, this dataset enables direct comparison 
 of model performance across all languages. Belebele opens up new avenues for evaluating and analyzing
  the multilingual abilities of language models and NLP systems.
"""

import os
from pathlib import Path
from typing import Dict, List, Tuple
import json
import datasets
import hashlib

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks, Licenses

_CITATION = """\
@article{,
  author    = {Lucas Bandarkar and Davis Liang and Benjamin Muller and Mikel Artetxe and Satya Narayan Shukla and Donald Husa and Naman Goyal and Abhinandan Krishnan and Luke Zettlemoyer and Madian Khabsa},
  title     = {The Belebele Benchmark: a Parallel Reading Comprehension Dataset in 122 Language Variants},
  journal   = {arXiv preprint arXiv:2308.16884},
  year      = {2023},
  url       = {https://arxiv.org/abs/2308.16884},
}
"""

_DATASETNAME = "belebele"

_DESCRIPTION = """\
Belebele is a multiple-choice machine reading comprehension (MRC) dataset spanning
 122 language variants. This dataset enables the evaluation of mono- and multi-lingual 
 models in high-, medium-, and low-resource languages. 
 Each question has four multiple-choice answers and is linked to a short passage 
 from the FLORES-200 dataset. The human annotation procedure was carefully curated 
 to create questions that discriminate between different levels of generalizable 
 language comprehension and is reinforced by extensive quality checks. 
 While all questions directly relate to the passage, the English dataset on its own 
 proves difficult enough to challenge state-of-the-art language models. 
 Being fully parallel, this dataset enables direct comparison of model performance 
 across all languages. Belebele opens up new avenues for evaluating and analyzing 
 the multilingual abilities of language models and NLP systems.
"""

_HOMEPAGE = "https://github.com/facebookresearch/belebele"

_LICENSE = Licenses.CC_BY_NC_SA_4_0.value

_URLS = {
    _DATASETNAME: "https://dl.fbaipublicfiles.com/belebele/Belebele.zip",
}

_SUPPORTED_TASKS = [Tasks.QUESTION_ANSWERING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

_SOURCE_NAMES = ["ceb_Latn", "ilo_Latn", "ind_Latn", "jav_Latn", "kac_Latn", "khm_Khmr", "lao_Laoo", "mya_Mymr", "shn_Mymr", "sun_Latn", "tgl_Latn", "tha_Thai", "vie_Latn", "war_Latn", "zsm_Latn"]
_LANGUAGES = [source.split("_")[0] for source in _SOURCE_NAMES]

_DEFAULT_LANG = "zsm"

_LOCAL = False

def config_constructor(belebele_subset: str, schema: str, version: str) -> SEACrowdConfig:
    lang = _LANGUAGES[_SOURCE_NAMES.index(belebele_subset)]
    return SEACrowdConfig(
        name="belebele_{belebele_subset}_{schema}".format(belebele_subset=belebele_subset.lower(), schema=schema),
        version=version,
        description="belebele {lang} {schema} schema".format(lang=lang, schema=schema),
        schema=schema,
        subset_id=lang,
    )

class BelebeleDataset(datasets.GeneratorBasedBuilder):
    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)
    BUILDER_CONFIGS = [config_constructor(lang, "source", _SOURCE_VERSION) for lang in _SOURCE_NAMES]
    BUILDER_CONFIGS.extend([config_constructor(source_subset, "seacrowd_qa", _SEACROWD_VERSION) for source_subset in _SOURCE_NAMES])

    #add config of "belebele_source" and "belebele_seacrowd_qa" for defined "_DEFAULT_LANG"
    BUILDER_CONFIGS.extend([
        SEACrowdConfig(
            name="belebele_source",
            version=_SOURCE_VERSION,
            description=f"belebele default source schema (using language of {_DEFAULT_LANG})",
            schema="source",
            subset_id=_DEFAULT_LANG
        ),
        SEACrowdConfig(
            name="belebele_seacrowd_qa",
            version=_SEACROWD_VERSION,
            description=f"belebele default seacrowd schema for QA task (using language of {_DEFAULT_LANG})",
            schema="seacrowd_qa",
            subset_id=_DEFAULT_LANG
        )]
    )
    DEFAULT_CONFIG_NAME = "belebele_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "link": datasets.Value("string"),
                    "question_number": datasets.Value("int64"),
                    "flores_passage": datasets.Value("string"),
                    "question": datasets.Value("string"),
                    "mc_answer1": datasets.Value("string"), 
                    "mc_answer2": datasets.Value("string"), 
                    "mc_answer3": datasets.Value("string"), 
                    "mc_answer4": datasets.Value("string"), 
                    "correct_answer_num": datasets.Value("string"), 
                    "dialect": datasets.Value("string"), 
                    "ds": datasets.Value("string"), # timedate
                }
            ) 
        elif self.config.schema == "seacrowd_qa":
            features = schemas.qa_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        source_name = _SOURCE_NAMES[_LANGUAGES.index(self.config.subset_id)]
        path = dl_manager.download_and_extract(_URLS[_DATASETNAME])
        file = "{path}/Belebele/{source_name}.jsonl".format(path=path, source_name=source_name)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "file": file,
                },
            ),
        ]

    def _generate_examples(self, file: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        if self.config.schema == "source":
            with open(file, "r", encoding="utf-8") as f: 
                for key, line in enumerate(f):
                    line = json.loads(line)
                    yield key, line
        elif self.config.schema == "seacrowd_qa":
            with open(file, "r", encoding="utf-8") as f: 
                for key, line in enumerate(f):
                    line = json.loads(line)
                    choices = [line['mc_answer1'], line['mc_answer2'], line['mc_answer3'], line['mc_answer4']]
                    answer = choices[int(line['correct_answer_num'])-1]
                    yield key, {
                        "id": key,
                        "question_id": str(line['question_number']),
                        "document_id": hashlib.md5(line['flores_passage'].encode('utf-8')).hexdigest(),
                        "question": line['question'],
                        "type": 'multiple_choice',
                        "choices": choices,
                        "context": line['flores_passage'],
                        "answer": [answer],
                        "meta": {}
                    }
        else:
            raise ValueError(f"Invalid config {self.config.name}")



from collections import defaultdict
from enum import Enum
from types import SimpleNamespace

from seacrowd.utils.schemas import (
    image_text_features,
    kb_features,
    tree_features,
    pairs_features,
    pairs_features_score,
    pairs_multi_features,
    qa_features,
    imqa_features,
    seq_label_features,
    speech2speech_features,
    speech_features,
    speech_multi_features,
    speech_text_features,
    ssp_features,
    text2text_features,
    text_features,
    text_multi_features,
    video_features,
    tod_features,
)

METADATA: dict = {
    "_LOCAL": bool,
    "_LANGUAGES": str,
    "_DISPLAYNAME": str,
}

SEACrowdValues = SimpleNamespace(NULL="<SEACROWD_NULL_STR>")

# Default View Name
DEFAULT_SOURCE_VIEW_NAME = "source"
DEFAULT_SEACROWD_VIEW_NAME = "seacrowd"


class Tasks(Enum):
    # Knowledge Base
    DEPENDENCY_PARSING = "DEP"
    KEYWORD_EXTRACTION = "KE"
    WORD_ANALOGY = "WA"
    WORD_SENSE_DISAMBIGUATION = "WSD"
    COREFERENCE_RESOLUTION = "COREF"
    RELATION_EXTRACTION = "RE"

    # Tree
    CONSTITUENCY_PARSING = "CONST_PAR"

    # Single Text Classification (single-label)
    ABUSIVE_LANGUAGE_PREDICTION = "ABL"
    COMPLAINT_DETECTION = "CD"
    DOMAIN_KNOWLEDGE_CLASSIFICATION = "DKC" # classification for non NLP-oriented label
    EMOTION_CLASSIFICATION = "EC"
    LANGUAGE_IDENTIFICATION = "LI"
    HOAX_NEWS_CLASSIFICATION = "HNC"
    INTENT_CLASSIFICATION = "INT"
    LEGAL_CLASSIFICATION = "LC"
    MORALITY_CLASSIFICATION = "MC"
    READABILITY_ASSESSMENT = "RA"
    RHETORIC_MODE_CLASSIFICATION = "RMC"
    SENTIMENT_ANALYSIS = "SA"
    TAX_COURT_VERDICT = "TACOS"
    TOPIC_MODELING = "TL"
    REINFORCEMENT_LEARNING_WITH_HUMAN_FEEDBACK = "RLHF"

    # Single Text Classification (multi-label)
    ASPECT_BASED_SENTIMENT_ANALYSIS = "ABSA"
    DOMAIN_KNOWLEDGE_MULTICLASSIFICATION = "DKM" # multi-classification for non NLP-oriented label
    CODE_SWITCHING_IDENTIFICATION = "CSI"

    # Single Text Sequence Labeling
    KEYWORD_TAGGING = "KT"
    NAMED_ENTITY_RECOGNITION = "NER"
    POS_TAGGING = "POS"
    SENTENCE_ORDERING = "SO"
    SLOT_FILLING = "SF"
    SPAN_BASED_ABSA = "SPAN_ABSA"
    TOKEN_LEVEL_LANGUAGE_IDENTIFICATION = "LANGID"

    # Pair Text Classification
    COMMONSENSE_REASONING = "CR"
    QUESTION_ANSWERING = "QA"
    TEXT_RETRIEVAL = "TRV"
    TEXTUAL_ENTAILMENT = "TE"
    SEMANTIC_SIMILARITY = "STS"
    NEXT_SENTENCE_PREDICTION = "NSP"
    SHORT_ANSWER_GRADING = "SAG"
    MORPHOLOGICAL_INFLECTION = "MOR"
    CONCEPT_ALIGNMENT_CLASSIFICATION = "CAC"

    # Single Text Generation
    CROSS_LINGUAL_SUMMARIZATION = "X-SUM"
    INSTRUCTION_TUNING = "ITT"
    MACHINE_TRANSLATION = "MT"
    MULTILEXNORM = "MLN"
    PARAPHRASING = "PARA"
    SUMMARIZATION = "SUM"
    TRANSLITERATION = "TRL"

    # Multi Text Generation
    DIALOGUE_SYSTEM = "DS"
    E2E_TASK_ORIENTED_DIALOGUE = "TOD"

    # Self Supervised & Unsupervised Text
    PROMPTING = "PRT"
    SELF_SUPERVISED_PRETRAINING = "SSP"

    # SpeechText
    SPEECH_RECOGNITION = "ASR"
    SPEECH_TO_TEXT_TRANSLATION = "STTT"

    SPEECH_LANGUAGE_IDENTIFICATION = "SPEECH_LID"
    SPEECH_EMOTION_RECOGNITION = "SER"
    SPEECH_EMOTION_RECOGNITION_MULTILABEL = "SER_MULTI"

    TEXT_TO_SPEECH = "TTS"

    # SpeechSpeech
    SPEECH_TO_SPEECH_TRANSLATION = "S2ST"

    # ImageText
    IMAGE_CAPTIONING = "IC"
    VISUAL_QUESTION_ANSWERING = "VQA"
    SIGN_LANGUAGE_RECOGNITION = "SLR"
    OPTICAL_CHARACTER_RECOGNITION = "OCR"
    STYLIZED_IMAGE_CAPTIONING = "SIC"
    VISUALLY_GROUNDED_REASONING = "VGR"

    # VideoText
    VIDEO_CAPTIONING = "VC"
    VIDEO_TO_TEXT_RETRIEVAL = "V2TR"

    # No seacrowd schema
    FACT_CHECKING = "FCT"
    WORD_LIST = "WL"


class Licenses(Enum):
    # BSD
    BSD = "BSD license family (bsd)"
    BSD_2_CLAUSE = "BSD 2-clause “Simplified” license (bsd-2-clause)"
    BSD_3_CLAUSE = "BSD 3-clause “New” or “Revised” license (bsd-3-clause)"
    BSD_3_CLAUSE_CLEAR = "BSD 3-clause Clear license (bsd-3-clause-clear)"

    # Creative Common
    CC = "Creative Commons license family (cc)"
    CC0_1_0 = "Creative Commons Zero v1.0 Universal (cc0-1.0)"
    CC_BY_2_0 = "Creative Commons Attribution 2.0 (cc-by-2.0)"
    CC_BY_2_5 = "Creative Commons Attribution 2.5 (cc-by-2.5)"
    CC_BY_3_0 = "Creative Commons Attribution 3.0 (cc-by-3.0)"
    CC_BY_4_0 = "Creative Commons Attribution 4.0 (cc-by-4.0)"
    CC_BY_SA_3_0 = "Creative Commons Attribution Share Alike 3.0 (cc-by-sa-3.0)"
    CC_BY_SA_4_0 = "Creative Commons Attribution Share Alike 4.0 (cc-by-sa-4.0)"
    CC_BY_NC_2_0 = "Creative Commons Attribution Non Commercial 2.0 (cc-by-nc-2.0)"
    CC_BY_NC_3_0 = "Creative Commons Attribution Non Commercial 3.0 (cc-by-nc-3.0)"
    CC_BY_NC_4_0 = "Creative Commons Attribution Non Commercial 4.0 (cc-by-nc-4.0)"
    CC_BY_ND_4_0 = "Creative Commons Attribution No Derivatives 4.0 (cc-by-nd-4.0)"
    CC_BY_NC_ND_3_0 = "Creative Commons Attribution Non Commercial No Derivatives 3.0 (cc-by-nc-nd-3.0)"
    CC_BY_NC_ND_4_0 = "Creative Commons Attribution Non Commercial No Derivatives 4.0 (cc-by-nc-nd-4.0)"
    CC_BY_NC_SA_2_0 = "Creative Commons Attribution Non Commercial Share Alike 2.0 (cc-by-nc-sa-2.0)"
    CC_BY_NC_SA_3_0 = "Creative Commons Attribution Non Commercial Share Alike 3.0 (cc-by-nc-sa-3.0)"
    CC_BY_NC_SA_4_0 = "Creative Commons Attribution Non Commercial Share Alike 4.0 (cc-by-nc-sa-4.0)"
    CDLA_SHARING_1_0 = "Community Data License Agreement – Sharing, Version 1.0 (cdla-sharing-1.0)"
    CDLA_PERMISSIVE_1_0 = "Community Data License Agreement – Permissive, Version 1.0 (cdla-permissive-1.0)"
    CDLA_PERMISSIVE_2_0 = "Community Data License Agreement – Permissive, Version 2.0 (cdla-permissive-2.0)"
    ECL_2_0 = "Educational Community License v2.0 (ecl-2.0)"
    WTFPL = "Do What The F*ck You Want To Public License (wtfpl)"

    # EPL
    EPL_1_0 = "Eclipse Public License 1.0 (epl-1.0)"
    EPL_2_0 = "Eclipse Public License 2.0 (epl-2.0)"
    EUPL_1_1 = "European Union Public License 1.1 (eupl-1.1)"

    # GPL
    AGPL_3_0 = "GNU Affero General Public License v3.0 (agpl-3.0)"
    GFDL = "GNU Free Documentation License family (gfdl)"
    GPL = "GNU General Public License family (gpl)"
    GPL_2_0 = "GNU General Public License v2.0 (gpl-2.0)"
    GPL_3_0 = "GNU General Public License v3.0 (gpl-3.0)"
    LGPL = "GNU Lesser General Public License family (lgpl)"
    LGPL_2_1 = "GNU Lesser General Public License v2.1 (lgpl-2.1)"
    LGPL_3_0 = "GNU Lesser General Public License v3.0 (lgpl-3.0)"
    LGPL_LR = "Lesser General Public License For Linguistic Resources (lgpl-lr)"

    # OTHER SPECIFIC LICENSES
    AFL_3_0 = "Academic Free License v3.0 (afl-3.0)"
    APACHE_2_0 = "Apache license 2.0 (apache-2.0)"
    ARTISTIC_2_0 = "Artistic license 2.0 (artistic-2.0)"
    BIGSCIENCE_OPENRAIL_M = "BigScience OpenRAIL-M (bigscience-openrail-m)"
    CREATIVEML_OPENRAIL_M = "CreativeML OpenRAIL-M (creativeml-openrail-m)"
    BIGSCIENCE_BLOOM_RAIL_1_0 = "BigScience BLOOM RAIL 1.0 (bigscience-bloom-rail-1.0)"
    BIGCODE_OPENRAIL_M = "BigCode Open RAIL-M v1 (bigcode-openrail-m)"
    BSL_1_0 = "Boost Software License 1.0 (bsl-1.0)"
    C_UDA = "Computational Use of Data Agreement (c-uda)"
    DEEPFLOYD_IF_LICENSE = "DeepFloyd IF Research License Agreement (deepfloyd-if-license)"
    ISC = "ISC (isc)"
    LLAMA2 = "Llama 2 Community License Agreement (llama2)"
    LPPL_1_3C = "LaTeX Project Public License v1.3c (lppl-1.3c)"
    MIT = "MIT (mit)"
    MS_PL = "Microsoft Public License (ms-pl)"
    MPL_2_0 = "Mozilla Public License 2.0 (mpl-2.0)"
    NCSA = "University of Illinois/NCSA Open Source License (ncsa)"
    ODC_BY = "Open Data Commons License Attribution family (odc-by)"
    ODBL = "Open Database License family (odbl)"
    OFL_1_1 = "SIL Open Font License 1.1 (ofl-1.1)"
    OPENRAIL = "OpenRAIL license family (openrail)"
    OPENRAIL_PP = "Open Rail++-M License (openrail++)"
    OSL_3_0 = "Open Software License 3.0 (osl-3.0)"
    PDDL = "Open Data Commons Public Domain Dedication and License (pddl)"
    POSTGRESQL = "PostgreSQL License (postgresql)"
    ZLIB = "zLib License (zlib)"

    # OTHER UNLISTED / UNLICENSED
    # for `OTHERS` license value, a terms of use of the data must be provided and accompanied by this LICENSE value:
    # e.g: f"{Licenses.OTHERS.value} | This data has terms of use of..."
    OTHERS = "Other License (others)"
    UNLICENSE = "The Unlicense (unlicense)"
    UNKNOWN = "Unknown (unknown)"


TASK_TO_SCHEMA = {
    Tasks.COREFERENCE_RESOLUTION: "KB",
    Tasks.DEPENDENCY_PARSING: "KB",
    Tasks.RELATION_EXTRACTION: "KB",
    Tasks.CONSTITUENCY_PARSING: "TREE",
    Tasks.E2E_TASK_ORIENTED_DIALOGUE: "TOD",
    Tasks.DIALOGUE_SYSTEM: "T2T",
    Tasks.WORD_SENSE_DISAMBIGUATION: "T2T",
    Tasks.WORD_ANALOGY: "T2T",
    Tasks.KEYWORD_EXTRACTION: "SEQ_LABEL",
    Tasks.KEYWORD_TAGGING: "SEQ_LABEL",
    Tasks.NAMED_ENTITY_RECOGNITION: "SEQ_LABEL",
    Tasks.POS_TAGGING: "SEQ_LABEL",
    Tasks.SENTENCE_ORDERING: "SEQ_LABEL",
    Tasks.SLOT_FILLING: "SEQ_LABEL",
    Tasks.SPAN_BASED_ABSA: "SEQ_LABEL",
    Tasks.TOKEN_LEVEL_LANGUAGE_IDENTIFICATION: "SEQ_LABEL",
    Tasks.COMMONSENSE_REASONING: "QA",
    Tasks.QUESTION_ANSWERING: "QA",
    Tasks.CONCEPT_ALIGNMENT_CLASSIFICATION: "PAIRS",
    Tasks.NEXT_SENTENCE_PREDICTION: "PAIRS",
    Tasks.TEXT_RETRIEVAL: "PAIRS",
    Tasks.TEXTUAL_ENTAILMENT: "PAIRS",
    Tasks.SEMANTIC_SIMILARITY: "PAIRS_SCORE",
    Tasks.SHORT_ANSWER_GRADING: "PAIRS_SCORE",
    Tasks.MORPHOLOGICAL_INFLECTION: "PAIRS_MULTI",
    Tasks.CROSS_LINGUAL_SUMMARIZATION: "T2T",
    Tasks.INSTRUCTION_TUNING: "T2T",
    Tasks.PARAPHRASING: "T2T",
    Tasks.MACHINE_TRANSLATION: "T2T",
    Tasks.MULTILEXNORM: "T2T",
    Tasks.SUMMARIZATION: "T2T",
    Tasks.TRANSLITERATION: "T2T",
    Tasks.ASPECT_BASED_SENTIMENT_ANALYSIS: "TEXT_MULTI",
    Tasks.DOMAIN_KNOWLEDGE_MULTICLASSIFICATION: "TEXT_MULTI",
    Tasks.CODE_SWITCHING_IDENTIFICATION: "TEXT_MULTI",
    Tasks.ABUSIVE_LANGUAGE_PREDICTION: "TEXT",
    Tasks.COMPLAINT_DETECTION: "TEXT",
    Tasks.DOMAIN_KNOWLEDGE_CLASSIFICATION: "TEXT",
    Tasks.SENTIMENT_ANALYSIS: "TEXT",
    Tasks.TAX_COURT_VERDICT: "TEXT",
    Tasks.EMOTION_CLASSIFICATION: "TEXT",
    Tasks.HOAX_NEWS_CLASSIFICATION: "TEXT",
    Tasks.LANGUAGE_IDENTIFICATION: "TEXT",
    Tasks.LEGAL_CLASSIFICATION: "TEXT",
    Tasks.INTENT_CLASSIFICATION: "TEXT",
    Tasks.READABILITY_ASSESSMENT: "TEXT",
    Tasks.RHETORIC_MODE_CLASSIFICATION: "TEXT",
    Tasks.TOPIC_MODELING: "TEXT",
    Tasks.REINFORCEMENT_LEARNING_WITH_HUMAN_FEEDBACK: "TEXT",
    Tasks.PROMPTING: "SSP",
    Tasks.SELF_SUPERVISED_PRETRAINING: "SSP",
    Tasks.SPEECH_RECOGNITION: "SPTEXT",
    Tasks.SPEECH_TO_TEXT_TRANSLATION: "SPTEXT",
    Tasks.TEXT_TO_SPEECH: "SPTEXT",
    Tasks.SPEECH_TO_SPEECH_TRANSLATION: "S2S",
    Tasks.SPEECH_LANGUAGE_IDENTIFICATION: "SPEECH",
    Tasks.SPEECH_EMOTION_RECOGNITION: "SPEECH",
    Tasks.SPEECH_EMOTION_RECOGNITION_MULTILABEL: "SPEECH_MULTI",
    Tasks.VISUAL_QUESTION_ANSWERING: "IMQA",
    Tasks.IMAGE_CAPTIONING: "IMTEXT",
    Tasks.SIGN_LANGUAGE_RECOGNITION: "IMTEXT",
    Tasks.OPTICAL_CHARACTER_RECOGNITION: "IMTEXT",
    Tasks.STYLIZED_IMAGE_CAPTIONING: "IMTEXT",
    Tasks.VISUALLY_GROUNDED_REASONING: "IMTEXT",
    Tasks.VIDEO_CAPTIONING: "VIDTEXT",
    Tasks.VIDEO_TO_TEXT_RETRIEVAL: "VIDTEXT",
    Tasks.FACT_CHECKING: None,
    Tasks.WORD_LIST: None,
    Tasks.MORALITY_CLASSIFICATION: "TEXT",
}

SCHEMA_TO_TASKS = defaultdict(set)
for task, schema in TASK_TO_SCHEMA.items():
    SCHEMA_TO_TASKS[schema].add(task)
SCHEMA_TO_TASKS = dict(SCHEMA_TO_TASKS)

VALID_TASKS = set(TASK_TO_SCHEMA.keys())
VALID_SCHEMAS = set(TASK_TO_SCHEMA.values())

SCHEMA_TO_FEATURES = {
    "KB": kb_features,
    "TREE": tree_features,
    "QA": qa_features,
    "T2T": text2text_features,
    "TEXT": text_features(),
    "TEXT_MULTI": text_multi_features(),
    "PAIRS": pairs_features(),
    "PAIRS_MULTI": pairs_multi_features(),
    "PAIRS_SCORE": pairs_features_score(),
    "SEQ_LABEL": seq_label_features(),
    "SSP": ssp_features,
    "SPTEXT": speech_text_features,
    "S2S": speech2speech_features,
    "SPEECH": speech_features(),
    "SPEECH_MULTI": speech_multi_features(),
    "IMTEXT": image_text_features(),
    "IMQA": imqa_features,
    "VIDTEXT": video_features,
    "TOD": tod_features,
}

TASK_TO_FEATURES = {
    Tasks.NAMED_ENTITY_RECOGNITION: {"entities"},
    Tasks.DEPENDENCY_PARSING: {"relations", "entities"},
    Tasks.COREFERENCE_RESOLUTION: {"entities", "coreferences"},
    Tasks.RELATION_EXTRACTION: {"relations", 'entities'},
    # Tasks.SPAN_BASED_ABSA: {"entities", "coreferences"},
    # Tasks.NAMED_ENTITY_DISAMBIGUATION: {"entities", "normalized"},
    # Tasks.EVENT_EXTRACTION: {"events"}
}



from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd
from datasets.download.download_manager import DownloadManager

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

# No paper citation found.
_CITATION = ""

_LOCAL = False
_LANGUAGES = ["tha"]
_DATASETNAME = "thai_databricks_dolly"
_DESCRIPTION = """\
This is a Thai-instructed dataset translated from databricks-dolly-15k using
Google Cloud Translation. databricks-dolly-15k is an open-source dataset of
instruction-following records generated by thousands of Databricks employees in
several behavioral categories outlined in the InstructGPT paper, including
brainstorming, classification, closed QA, generation, information extraction,
open QA, and summarization.
"""

_HOMEPAGE = "https://huggingface.co/datasets/Thaweewat/databricks-dolly-15k-th"
_LICENSE = Licenses.CC_BY_SA_3_0.value
_URL = "https://huggingface.co/datasets/Thaweewat/databricks-dolly-15k-th/resolve/main/databricks-dolly-15k-th.parquet"
_SUPPORTED_TASKS = [Tasks.INSTRUCTION_TUNING]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class ThaiDatabricksDollyDataset(datasets.GeneratorBasedBuilder):
    """Thai Databricks Dolly Dataset"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = "t2t"

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "instruction": datasets.Value("string"),
                    "context": datasets.Value("string"),
                    "response": datasets.Value("string"),
                    "category": datasets.Value("string"),
                }
            )
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.text2text_features
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        data_file = Path(dl_manager.download_and_extract(_URL))
        return [datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": data_file})]

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        """Yield examples as (key, example) tuples"""
        # pyarrow is an implicit dependency to load the parquet files
        df = pd.read_parquet(filepath, engine="pyarrow")
        for idx, row in df.iterrows():
            instruction = row.get("instruction").strip()
            context = row.get("context").strip()
            response = row.get("response").strip()
            category = row.get("category").strip()
            if self.config.schema == "source":
                example = {
                    "instruction": instruction,
                    "context": context,
                    "response": response,
                    "category": category,
                }
            elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
                text_1 = f"Context: {context}\n\n{instruction}" if context else instruction
                text_2 = response
                example = {
                    "id": str(idx),
                    "text_1": text_1,
                    "text_2": text_2,
                    "text_1_name": "context_and_instruction",
                    "text_2_name": "response",
                }

            yield idx, example



from pathlib import Path
from typing import List

import datasets
import json

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks, DEFAULT_SOURCE_VIEW_NAME, DEFAULT_SEACROWD_VIEW_NAME

_DATASETNAME = "news_en_id"
_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME

_LANGUAGES = ["ind", "eng"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False
_CITATION = """\
@inproceedings{guntara-etal-2020-benchmarking,
    title = "Benchmarking Multidomain {E}nglish-{I}ndonesian Machine Translation",
    author = "Guntara, Tri Wahyu  and
      Aji, Alham Fikri  and
      Prasojo, Radityo Eko",
    booktitle = "Proceedings of the 13th Workshop on Building and Using Comparable Corpora",
    month = may,
    year = "2020",
    address = "Marseille, France",
    publisher = "European Language Resources Association",
    url = "https://aclanthology.org/2020.bucc-1.6",
    pages = "35--43",
    language = "English",
    ISBN = "979-10-95546-42-9",
}
"""

_DESCRIPTION = """\
News En-Id is a machine translation dataset containing Indonesian-English parallel sentences collected from the news. The news dataset is collected from multiple sources: Pan Asia Networking Localization (PANL), Bilingual BBC news articles, Berita Jakarta, and GlobalVoices. We split the dataset and use 75% as the training set, 10% as the validation set, and 15% as the test set. Each of the datasets is evaluated in both directions, i.e., English to Indonesian (En → Id) and Indonesian to English (Id → En) translations.
"""

_HOMEPAGE = "https://github.com/gunnxx/indonesian-mt-data"

_LICENSE = "Creative Commons Attribution Share-Alike 4.0 International"

_URLs = {"indonlg": "https://storage.googleapis.com/babert-pretraining/IndoNLG_finals/downstream_task/downstream_task_datasets.zip"}

_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class NewsEnId(datasets.GeneratorBasedBuilder):
    """Bible Su-Id is a machine translation dataset containing Indonesian-Sundanese parallel sentences collected from the bible.."""

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="news_en_id_source",
            version=datasets.Version(_SOURCE_VERSION),
            description="News En-Id source schema",
            schema="source",
            subset_id="news_en_id",
        ),
        SEACrowdConfig(
            name="news_en_id_seacrowd_t2t",
            version=datasets.Version(_SEACROWD_VERSION),
            description="News En-Id Nusantara schema",
            schema="seacrowd_t2t",
            subset_id="news_en_id",
        ),
    ]

    DEFAULT_CONFIG_NAME = "news_en_id_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features({"id": datasets.Value("string"), "text": datasets.Value("string"), "label": datasets.Value("string")})
        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        base_path = Path(dl_manager.download_and_extract(_URLs["indonlg"])) / "IndoNLG_downstream_tasks" / "MT_IMD_NEWS"
        data_files = {
            "train": base_path / "train_preprocess.json",
            "validation": base_path / "valid_preprocess.json",
            "test": base_path / "test_preprocess.json",
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_files["train"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": data_files["validation"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": data_files["test"]},
            ),
        ]

    def _generate_examples(self, filepath: Path):
        data = json.load(open(filepath, "r"))
        if self.config.schema == "source":
            for row in data:
                ex = {"id": row["id"], "text": row["text"], "label": row["label"]}
                yield row["id"], ex
        elif self.config.schema == "seacrowd_t2t":
            for row in data:
                ex = {
                    "id": row["id"],
                    "text_1": row["text"],
                    "text_2": row["label"],
                    "text_1_name": "eng",
                    "text_2_name": "ind",
                }
                yield row["id"], ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



from pathlib import Path
from typing import Dict, List, Tuple
import re


import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import DEFAULT_SEACROWD_VIEW_NAME, DEFAULT_SOURCE_VIEW_NAME, Tasks

_DATASETNAME = "nusatranslation_mt"
_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME

_LANGUAGES = ["ind", "btk", "bew", "bug", "jav", "mad", "mak", "min", "mui", "rej", "sun"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False

_CITATION = """\
    @unpublished{anonymous2023nusawrites:,
    title={NusaWrites: Constructing High-Quality Corpora for Underrepresented and Extremely Low-Resource Languages},
    author={Anonymous},
    journal={OpenReview Preprint},
    year={2023},
    note={anonymous preprint under review}
    }
    """

_DESCRIPTION = """\
    Democratizing access to natural language processing (NLP) technology is crucial, especially for underrepresented and extremely low-resource languages. Previous research has focused on developing labeled and unlabeled corpora for these languages through online scraping and document translation. While these methods have proven effective and cost-efficient, we have identified limitations in the resulting corpora, including a lack of lexical diversity and cultural relevance to local communities. To address this gap, we conduct a case study on Indonesian local languages. We compare the effectiveness of online scraping, human translation, and paragraph writing by native speakers in constructing datasets. Our findings demonstrate that datasets generated through paragraph writing by native speakers exhibit superior quality in terms of lexical diversity and cultural content. In addition, we present the NusaWrites benchmark, encompassing 12 underrepresented and extremely low-resource languages spoken by millions of individuals in Indonesia. Our empirical experiment results using existing multilingual large language models conclude the need to extend these models to more underrepresented languages.
    We introduce a novel high quality human curated corpora, i.e., NusaMenulis, which covers 12 languages spoken in Indonesia. The resource extend the coverage of languages to 5 new languages, i.e., Ambon (abs), Bima (bhp), Makassarese (mak), Palembang / Musi (mui), and Rejang (rej).
    For the rhetoric mode classification task, we cover 5 rhetoric modes, i.e., narrative, persuasive, argumentative, descriptive, and expository.
"""

_HOMEPAGE = "https://github.com/IndoNLP/nusatranslation/tree/main/datasets/mt"

_LICENSE = "Creative Commons Attribution Share-Alike 4.0 International"

_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

_URLS = {
    "train": "https://raw.githubusercontent.com/IndoNLP/nusa-writes/main/data/nusa_kalimat-mt-{lang}-train.csv",
    "validation": "https://raw.githubusercontent.com/IndoNLP/nusa-writes/main/data/nusa_kalimat-mt-{lang}-valid.csv",
    "test": "https://raw.githubusercontent.com/IndoNLP/nusa-writes/main/data/nusa_kalimat-mt-{lang}-test.csv",
}

LANGUAGES_MAP = {
    "abs": "ambon",
    "btk": "batak",
    "bew": "betawi",
    "bhp": "bima",
    "jav": "javanese",
    "mad": "madurese",
    "mak": "makassarese",
    "min": "minangkabau",
    "mui": "musi",
    "rej": "rejang",
    "sun": "sundanese",
}


class NusaTranslationMT(datasets.GeneratorBasedBuilder):
    """NusaTranslation-MT is a parallel corpus for training and benchmarking machine translation models from 11 Indonesian local language to Bahasa Indonesia. The data is presented in csv format with 2 columns, where one column contain sentence in Bahasa and another in the local language."""

    BUILDER_CONFIGS = (
        [
            SEACrowdConfig(
                name=f"nusatranslation_mt_ind_{subset}_source",
                version=datasets.Version(_SOURCE_VERSION),
                description=f"nusatranslation_mt ind2{subset} source schema",
                schema="source",
                subset_id=f"nusatranslation_mt",
            )
            for subset in _LANGUAGES[1:]
        ]
        + [
            SEACrowdConfig(
                name=f"nusatranslation_mt_ind_{subset}_seacrowd_t2t",
                version=datasets.Version(_SEACROWD_VERSION),
                description=f"nusatranslation_mt ind2{subset} Nusantara schema",
                schema="seacrowd_t2t",
                subset_id=f"nusatranslation_mt",
            )
            for subset in _LANGUAGES[1:]
        ]
        + [
            SEACrowdConfig(
                name=f"nusatranslation_mt_{subset}_ind_source",
                version=datasets.Version(_SOURCE_VERSION),
                description=f"nusatranslation_mt {subset}2ind source schema",
                schema="source",
                subset_id=f"nusatranslation_mt",
            )
            for subset in _LANGUAGES[1:]
        ]
        + [
            SEACrowdConfig(
                name=f"nusatranslation_mt_{subset}_ind_seacrowd_t2t",
                version=datasets.Version(_SEACROWD_VERSION),
                description=f"nusatranslation_mt {subset}2ind Nusantara schema",
                schema="seacrowd_t2t",
                subset_id=f"nusatranslation_mt",
            )
            for subset in _LANGUAGES[1:]
        ]
    )

    DEFAULT_CONFIG_NAME = "nusatranslation_mt_jav_ind_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features({"id": datasets.Value("string"), "text": datasets.Value("string"), "label": datasets.Value("string")})
        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        lang = self.config.name.split("_")[2] if self.config.name.split("_")[2] != "ind" else self.config.name.split("_")[3]
        train_csv_path = Path(dl_manager.download_and_extract(_URLS["train"].format(lang=lang)))
        validation_csv_path = Path(dl_manager.download_and_extract(_URLS["validation"].format(lang=lang)))
        test_csv_path = Path(dl_manager.download_and_extract(_URLS["test"].format(lang=lang)))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": train_csv_path},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": validation_csv_path},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": test_csv_path},
            ),
        ]

    def _merge_subsets(self, df, subsets, revert=False):
        if not subsets:
            return None
        # df = None
        # print(dfs)
        # print(subsets)
        orig_columns = df.columns.tolist()
        print(df.columns)

        df.columns = orig_columns[:1] + ["label", "text"] if revert else orig_columns[:1] + ["text", "label"]
        return df

    def get_domain_data(self, dfs):
        domain = self.config.name
        matched_domain = re.findall(r"nusatranslation_mt_.*?_.*?_", domain)

        assert len(matched_domain) == 1
        domain = matched_domain[0][:-1].replace("nusatranslation_mt_", "").split("_")
        src_lang, tgt_lang = domain[0], domain[1]

        subsets = LANGUAGES_MAP.get(src_lang if src_lang != "ind" else tgt_lang, None)
        return src_lang, tgt_lang, self._merge_subsets(dfs, subsets, revert=(src_lang != "ind"))

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        if self.config.schema != "source" and self.config.schema != "seacrowd_t2t":
            raise ValueError(f"Invalid config schema: {self.config.schema}")

        # print(filepath)
        df = pd.read_csv(filepath)
        # ldf = []
        # for fp in filepath:
        #     ldf.append(pd.read_csv(fp))
        src_lang, tgt_lang, df = self.get_domain_data((df))

        if self.config.schema == "source":
            for idx, row in enumerate(df.itertuples()):
                ex = {
                    "id": str(idx),
                    "text": row.text,
                    "label": row.label,
                }
                yield idx, ex

        elif self.config.schema == "seacrowd_t2t":
            for idx, row in enumerate(df.itertuples()):
                ex = {
                    "id": str(idx),
                    "text_1": row.text,
                    "text_2": row.label,
                    "text_1_name": src_lang,
                    "text_2_name": tgt_lang,
                }
                yield idx, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



import re


def extract_parts(input_string):
    parts = []
    stack = []
    current_part = ""

    for char in input_string:
        if char == "(":
            stack.append("(")
        elif char == ")":
            if stack:
                stack.pop()
                if not stack:
                    parts.append(current_part[1:].strip())
                    current_part = ""
            else:
                parts.append(current_part[1:].strip())
                current_part = ""
        if stack:
            current_part += char

    return parts


def extract_sentence(input_string):
    innermost_pattern = re.compile(r"\(([^()]+)\)")
    innermost_matches = re.findall(innermost_pattern, input_string)
    extracted_sentence = " ".join(match.split()[1] for match in innermost_matches)
    if len(extracted_sentence) == 0:
        extracted_sentence = " ".join(input_string.split()[1:])
    return extracted_sentence


def extract_data(sentence):
    nodes = []
    sub_nodes = {}
    sub_node_ids = []

    # Extract id, sub_nodes and text of ROOT
    sentence_id = sentence.split("\t")[0]
    root_sent = sentence[sentence.find("ROOT") : -1]
    root_subnodes = extract_parts(root_sent)
    sub_nodes.update({i + 1: root_subnodes[i] for i in range(len(root_subnodes))})
    sub_node_ids.extend([i + 1 for i in range(len(root_subnodes))])
    root_text = extract_sentence(root_sent)

    nodes.append({"id": f"{sentence_id+'.'+str(0)}", "type": "ROOT", "text": root_text, "offsets": [0, len(root_text) - 1], "subnodes": [f"{sentence_id+'.'+str(i)}" for i in sub_node_ids]})

    while sub_node_ids:
        sub_node_id = sub_node_ids.pop(0)
        text = extract_sentence(sub_nodes[sub_node_id])

        cur_subnodes = extract_parts(sub_nodes[sub_node_id])

        if len(cur_subnodes) > 0:
            id_to_add = sub_node_ids[-1] if len(sub_node_ids) > 0 else sub_node_id
            cur_subnode_ids = [id_to_add + i + 1 for i in range(len(cur_subnodes))]
            sub_nodes.update({id_to_add + i + 1: cur_subnodes[i] for i in range(len(cur_subnodes))})
            sub_node_ids.extend(cur_subnode_ids)
        else:
            cur_subnode_ids = []

        node_type = sub_nodes[sub_node_id].split(" ")[0]
        start = root_text.find(text)
        end = start + len(text) - 1

        nodes.append({"id": f"{sentence_id+'.'+str(sub_node_id)}", "type": node_type, "text": text, "offsets": [start, end], "subnodes": [f"{sentence_id+'.'+str(i)}" for i in cur_subnode_ids]})
    return {"id": sentence_id, "passage": {"id": sentence_id + "_0", "type": None, "text": [nodes[0]["text"]], "offsets": nodes[0]["offsets"]}, "nodes": nodes}



import re
import gzip
import json
from pathlib import Path
from typing import Dict, List, Tuple
from urllib.parse import urljoin

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks, Licenses

_CITATION = """\
@inproceedings{abadji2022cleaner,
    author    = {Julien Abadji and
                Pedro Javier Ortiz Su{\'{a}}rez and
                Laurent Romary and
                Beno{\^{\i}}t Sagot},
    title     = {Towards a Cleaner Document-Oriented Multilingual Crawled Corpus},
    booktitle = {Proceedings of the Thirteenth Language Resources and Evaluation Conference,
                {LREC} 2022, Marseille, France, 20-25 June 2022},
    pages     = {4344--4355},
    publisher = {European Language Resources Association},
    year      = {2022},
    url       = {https://aclanthology.org/2022.lrec-1.463},
}

@inproceedings{abadji2021ungoliant,
    author    = {Julien Abadji and
                Pedro Javier Ortiz Su{\'a}rez and
                Laurent Romary and
                Beno{\^i}t Sagot},
    title     = {Ungoliant: An optimized pipeline for the generation of a very large-scale multilingual web corpus},
    series    = {Proceedings of the Workshop on Challenges in the Management of Large Corpora
                (CMLC-9) 2021. Limerick, 12 July 2021 (Online-Event)},
    editor    = {Harald L{\"u}ngen and
                Marc Kupietz and
                Piotr Bański and
                Adrien Barbaresi and
                Simon Clematide and
                Ines Pisetta},
    publisher = {Leibniz-Institut f{\"u}r Deutsche Sprache},
    address   = {Mannheim},
    doi       = {10.14618/ids-pub-10468},
    url       = {https://nbn-resolving.org/urn:nbn:de:bsz:mh39-104688},
    pages     = {1 -- 9},
    year      = {2021},
    abstract  = {Since the introduction of large language models in Natural Language
    Processing, large raw corpora have played a crucial role in Computational Linguistics.
    However, most of these large raw corpora are either available only for English or not
    available to the general public due to copyright issues. Nevertheless, there are some
    examples of freely available multilingual corpora for training Deep Learning NLP
    models, such as the OSCAR and Paracrawl corpora. However, they have quality issues,
    especially for low-resource languages. Moreover, recreating or updating these corpora
    is very complex. In this work, we try to reproduce and improve the goclassy pipeline
    used to create the OSCAR corpus. We propose a new pipeline that is faster, modular,
    parameterizable, and well documented. We use it to create a corpus similar to OSCAR
    but larger and based on recent data. Also, unlike OSCAR, the metadata information is
    at the document level. We release our pipeline under an open source license and
    publish the corpus under a research-only license.},
    language  = {en}
}

@article{kreutzer2022quality,
    title     = {Quality at a Glance: An Audit of Web-Crawled Multilingual Datasets},
    author    = {Kreutzer, Julia  and
                Caswell, Isaac  and
                Wang, Lisa  and
                Wahab, Ahsan  and
                van Esch, Daan  and
                Ulzii-Orshikh, Nasanbayar  and
                Tapo, Allahsera  and
                Subramani, Nishant  and
                Sokolov, Artem  and
                Sikasote, Claytone  and
                Setyawan, Monang  and
                Sarin, Supheakmungkol  and
                Samb, Sokhar  and
                Sagot, Beno{\^\i}t  and
                Rivera, Clara  and
                Rios, Annette  and
                Papadimitriou, Isabel  and
                Osei, Salomey  and
                Suarez, Pedro Ortiz  and
                Orife, Iroro  and
                Ogueji, Kelechi  and
                Rubungo, Andre Niyongabo  and
                Nguyen, Toan Q.  and
                M{\"u}ller, Mathias  and
                M{\"u}ller, Andr{\'e}  and
                Muhammad, Shamsuddeen Hassan  and
                Muhammad, Nanda  and
                Mnyakeni, Ayanda  and
                Mirzakhalov, Jamshidbek  and
                Matangira, Tapiwanashe  and
                Leong, Colin  and
                Lawson, Nze  and
                Kudugunta, Sneha  and
                Jernite, Yacine  and
                Jenny, Mathias  and
                Firat, Orhan  and
                Dossou, Bonaventure F. P.  and
                Dlamini, Sakhile  and
                de Silva, Nisansa  and
                {\c{C}}abuk Ball{\i}, Sakine  and
                Biderman, Stella  and
                Battisti, Alessia  and
                Baruwa, Ahmed  and
                Bapna, Ankur  and
                Baljekar, Pallavi  and
                Azime, Israel Abebe  and
                Awokoya, Ayodele  and
                Ataman, Duygu  and
                Ahia, Orevaoghene  and
                Ahia, Oghenefego  and
                Agrawal, Sweta  and
                Adeyemi, Mofetoluwa},
    editor    = {Roark, Brian  and
                Nenkova, Ani},
    journal   = {Transactions of the Association for Computational Linguistics},
    volume    = {10},
    year      = {2022},
    address   = {Cambridge, MA},
    publisher = {MIT Press},
    url       = {https://aclanthology.org/2022.tacl-1.4},
    doi       = {10.1162/tacl_a_00447},
    pages     = {50--72},
    abstract  = {With the success of large-scale pre-training and multilingual modeling in
    Natural Language Processing (NLP), recent years have seen a proliferation of large,
    Web-mined text datasets covering hundreds of languages. We manually audit the quality
    of 205 language-specific corpora released with five major public datasets (CCAligned,
    ParaCrawl, WikiMatrix, OSCAR, mC4). Lower-resource corpora have systematic issues: At
    least 15 corpora have no usable text, and a significant fraction contains less than
    50{\%} sentences of acceptable quality. In addition, many are mislabeled or use
    nonstandard/ambiguous language codes. We demonstrate that these issues are easy to
    detect even for non-proficient speakers, and supplement the human audit with automatic
    analyses. Finally, we recommend techniques to evaluate and improve multilingual
    corpora and discuss potential risks that come with low-quality data releases.},
}

@inproceedings{ortizsuarez2020monolingual,
    title     = {A Monolingual Approach to Contextualized Word Embeddings for Mid-Resource Languages},
    author    = {Ortiz Su{'a}rez, Pedro Javier  and
                Romary, Laurent  and
                Sagot, Benoit},
    booktitle = {Proceedings of the 58th Annual Meeting of the Association for Computational Linguistics},
    month     = {jul},
    year      = {2020},
    address   = {Online},
    publisher = {Association for Computational Linguistics},
    url       = {https://www.aclweb.org/anthology/2020.acl-main.156},
    pages     = {1703--1714},
    abstract  = {We use the multilingual OSCAR corpus, extracted from Common Crawl via
    language classification, filtering and cleaning, to train monolingual contextualized
    word embeddings (ELMo) for five mid-resource languages. We then compare the
    performance of OSCAR-based and Wikipedia-based ELMo embeddings for these languages on
    the part-of-speech tagging and parsing tasks. We show that, despite the noise in the
    Common-Crawl-based OSCAR data, embeddings trained on OSCAR perform much better than
    monolingual embeddings trained on Wikipedia. They actually equal or improve the
    current state of the art in tagging and parsing for all five languages. In particular,
    they also improve over multilingual Wikipedia-based contextual embeddings
    (multilingual BERT), which almost always constitutes the previous state of the art,
    thereby showing that the benefit of a larger, more diverse corpus surpasses the
    cross-lingual benefit of multilingual embedding architectures.},
}

@inproceedings{ortizsuarez2019asynchronous,
    author    = {Pedro Javier {Ortiz Su{'a}rez} and
                Benoit Sagot and
                Laurent Romary},
    title     = {Asynchronous pipelines for processing huge corpora on medium to low resource infrastructures},
    series    = {Proceedings of the Workshop on Challenges in the Management of Large Corpora
                (CMLC-7) 2019. Cardiff, 22nd July 2019},
    editor    = {Piotr Bański and
                Adrien Barbaresi and
                Hanno Biber and
                Evelyn Breiteneder and
                Simon Clematide and
                Marc Kupietz and
                Harald L{"u}ngen and
                Caroline Iliadi},
    publisher = {Leibniz-Institut f{"u}r Deutsche Sprache},
    address   = {Mannheim},
    doi       = {10.14618/ids-pub-9021},
    url       = {http://nbn-resolving.de/urn:nbn:de:bsz:mh39-90215},
    pages     = {9 -- 16},
    year      = {2019},
    abstract  = {Common Crawl is a considerably large, heterogeneous multilingual corpus
    comprised of crawled documents from the internet, surpassing 20TB of data and
    distributed as a set of more than 50 thousand plain text files where each contains
    many documents written in a wide variety of languages. Even though each document has a
    metadata block associated to it, this data lacks any information about the language in
    which each document is written, making it extremely difficult to use Common Crawl for
    monolingual applications. We propose a general, highly parallel, multithreaded
    pipeline to clean and classify Common Crawl by language; we specifically design it so
    that it runs efficiently on medium to low resource infrastructures where I/O speeds
    are the main constraint. We develop the pipeline so that it can be easily reapplied to
    any kind of heterogeneous corpus and so that it can be parameterised to a wide range
    of infrastructures. We also distribute a 6.3TB version of Common Crawl, filtered,
    classified by language, shuffled at line level in order to avoid copyright issues, and
    ready to be used for NLP applications.},
    language  = {en}
}
"""

_DATASETNAME = "oscar_2201"
_DESCRIPTION = """\
OSCAR or Open Super-large Crawled Aggregated coRpus is a huge multilingual corpus
obtained by language classification and filtering of the Common Crawl corpus using
the ungoliant architecture. Data is distributed by language in both original and
deduplicated form.
"""

_HOMEPAGE = "https://huggingface.co/datasets/oscar-corpus/OSCAR-2201"
_LICENSE = Licenses.CC0_1_0.value
_BASE_URL = "https://huggingface.co/datasets/oscar-corpus/OSCAR-2201/resolve/main/compressed/{lang}_meta/"

_LOCAL = False
_LANGUAGES = ["war", "ceb", "min", "vie", "ilo", "tgl", "lao", "khm", "mya", "jav", "ind", "tha", "sun", "zlm"]

_SUPPORTED_TASKS = [Tasks.SELF_SUPERVISED_PRETRAINING]
_SOURCE_VERSION = "2022.1.0"
_SEACROWD_VERSION = "1.0.0"


class Oscar2201Dataset(datasets.GeneratorBasedBuilder):
    """OSCAR subset for SEA languages, version 2201."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = "ssp"
    SUBSETS = ["war", "ceb", "min", "vi", "ta", "ilo", "tl", "lo", "km", "my", "jv", "id", "th", "su", "ms"]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{subset}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME} {subset} source schema",
            schema="source",
            subset_id=subset,
        ) for subset in SUBSETS
    ] + [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{subset}_seacrowd_ssp",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME} {subset} SEACrowd schema",
            schema="seacrowd_ssp",
            subset_id=subset,
        )
        for subset in SUBSETS
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_jv_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("int64"),
                    "text": datasets.Value("string"),
                    "meta": {
                        "warc_headers": {
                            "warc-record-id": datasets.Value("string"),
                            "warc-date": datasets.Value("string"),
                            "content-type": datasets.Value("string"),
                            "content-length": datasets.Value("int32"),
                            "warc-type": datasets.Value("string"),
                            "warc-identified-content-language": datasets.Value("string"),
                            "warc-refers-to": datasets.Value("string"),
                            "warc-target-uri": datasets.Value("string"),
                            "warc-block-digest": datasets.Value("string"),
                        },
                        "identification": {
                            "label": datasets.Value("string"),
                            "prob": datasets.Value("float"),
                        },
                        "annotations": datasets.Sequence(datasets.Value("string")),
                        "line_identifications": [
                            {
                                "label": datasets.Value("string"),
                                "prob": datasets.Value("float"),
                            }
                        ],
                    },
                }
            )
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.ssp_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        base_path = _BASE_URL.format(lang=self.config.name.split("_")[2])

        checksum_url = urljoin(base_path, "checksum.sha256")
        checksum_path = Path(dl_manager.download(checksum_url))
        with open(checksum_path, encoding="utf-8") as f:
            filenames = [line.split()[1] for line in f if line]
            filenames = sorted(filenames, key=lambda x: int(re.search(r"\d+", x).group()) if re.search(r"\d+", x) else x)
            data_urls = [urljoin(base_path, filename) for filename in filenames]

        data_paths = list(map(Path, dl_manager.download([url for url in data_urls if url.endswith(".jsonl.gz")])))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepaths": data_paths,
                    "split": "train",
                },
            )
        ]

    def _generate_examples(self, filepaths: [Path], split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        key = 0
        for filepath in filepaths:
            with gzip.open(open(filepath, "rb"), "rt", encoding="utf-8") as f:
                for line in f:
                    doc = json.loads(line)
                    if self.config.schema == "source":
                        meta = dict()
                        meta["warc_headers"] = doc["warc_headers"]
                        meta["warc_headers"]["warc-identified-content-language"] = doc["warc_headers"].get("warc-identified-content-language")
                        meta["identification"] = doc["metadata"]["identification"]
                        meta["annotations"] = doc["metadata"]["annotation"]
                        meta["line_identifications"] = doc["metadata"]["sentence_identifications"]
                        yield key, {"id": key, "text": doc["content"], "meta": meta}
                    elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
                        yield key, {"id": str(key), "text": doc["content"]}
                    key += 1


# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pathlib import Path
from typing import Dict, List, Tuple
from seacrowd.utils.constants import Tasks
from seacrowd.utils import schemas

import datasets

from seacrowd.utils.configs import SEACrowdConfig

# TODO: Add BibTeX citation
_CITATION = """\
@inproceedings{siallagan2022sampiran,
  title={Poetry Generation for Indonesian Pantun: Comparison Between SeqGAN and GPT-2},
  author={Emmanuella Anggi Siallagan and Ika Alfina},
  booktitle={Jurnal Ilmu Komputer dan Informasi (Journal of Computer Science and Information) Vol 1x No x February 2023 (Minor Revision)},
  year={2023},
}
"""

_DATASETNAME = "sampiran"


_DESCRIPTION = """\
Sampiran is a dataset for pantun generation. It consists of 7.8K Indonesian pantun, collected from various sources (online). 
Pantun is a traditional Malay poem consisting of four lines: two lines of deliverance and two lines of message. 
This dataset filtered the gathered Pantun to follow the general rules of Pantun; four lines with ABAB rhyme and eight to twelve syllables per line.
"""

_LANGUAGES = ["ind"]
_LOCAL = False
_HOMEPAGE = "https://github.com/ir-nlp-csui/sampiran"
_LICENSE = "AGPL-3.0"

_URLS = "https://raw.githubusercontent.com/ir-nlp-csui/sampiran/main/sampiran.txt"

_SUPPORTED_TASKS = [Tasks.SELF_SUPERVISED_PRETRAINING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class SampiranDataset(datasets.GeneratorBasedBuilder):
    """Sampiran is a dataset for pantun generation. It consists of 7.8K Indonesian pantun,
    collected from various sources (online)."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="sampiran_source",
            version=SOURCE_VERSION,
            description="sampiran source schema",
            schema="source",
            subset_id="sampiran",
        ),
        SEACrowdConfig(
            name="sampiran_seacrowd_ssp",
            version=SEACROWD_VERSION,
            description="sampiran Nusantara schema",
            schema="seacrowd_ssp",
            subset_id="sampiran",
        ),
    ]

    DEFAULT_CONFIG_NAME = "sampiran_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "pantun": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_ssp":
            # e.g. features = schemas.kb_features
            # TODO: Choose your seacrowd schema here
            features = schemas.self_supervised_pretraining.features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(
        self, dl_manager: datasets.DownloadManager
    ) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        filepath = Path(dl_manager.download(_URLS))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": filepath},
            ),
        ]

    def _read_data(self, filepath: Path) -> List[Dict]:
        """Reads the data from the source file and returns a list of dicts."""

    def _generate_examples(self, filepath: Path, split: str = None) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        if self.config.schema != "source" and self.config.schema != "seacrowd_ssp":
            raise ValueError(f"Invalid config schema: {self.config.schema}")

        # Read the file line by line

        if self.config.name == "sampiran_source":
            with open(filepath, encoding="utf-8") as f:
                for id_, row in enumerate(f):
                    ex = {
                        "id": str(id_),
                        "pantun": str(row).rstrip(),
                    }
                    yield id_, ex

        elif self.config.name == "sampiran_seacrowd_ssp":
            with open(filepath, encoding="utf-8") as f:
                for id_, row in enumerate(f):
                    ex = {"id": str(id_), "text": str(row).rstrip()}
                    yield id_, ex
